/*
 * @file ucd9081_interface.h
 *
 * @brief This file contains structures,function prototype to read and
 * store the data from voltage sequencer
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DRIVERS_SENSOR_INTERFACE_VOLTAGE_SEQUENCER_INC_UCD9081_INTERFACE_H_
#define DRIVERS_SENSOR_INTERFACE_VOLTAGE_SEQUENCER_INC_UCD9081_INTERFACE_H_

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include "exo_ahw_al_voltage_sequencer_common.h"
#include "exo_osal_timer.h"
#include "sensor_driver_intf.h"

/**
 * @brief Voltage sequencer data structure definition
 */
typedef struct
{
	float voltage;					/*!< voltage						*/
	vltg_seq_rail_num rail_no;		/*!< RTC time structure variable	*/

}__attribute__((packed, aligned(4)))vsm_get_info_s;

/**
 * @brief Voltage sequencer data unit structure definition
 */
typedef struct
{
	vsm_get_info_s vol;				/*!< Voltage sequencer data structure variable	        */
	uint64_t vsm_epoch_time;		/*!< RTC time structure variable						*/

}__attribute__((packed, aligned(4)))vsm_data_s;


/**
 * @brief  API to get the voltage sequencer data
 * @param vsm_param - pointer to a voltage sequencer data structure variable
 * @param vsm_inst_id - voltage sequencer instance ID
 * @retval No return
 */
void vsm_get_func(vsm_get_info_s *vsm_param,uint8_t vsm_inst_id);


#endif /* DRIVERS_SENSOR_INTERFACE_VOLTAGE_SEQUENCER_INC_UCD9081_INTERFACE_H_ */
