/*
 * @file ds620_interface.h
 *
 * @brief This file contains structures,function prototype to read and
 * store the data from digital thermostat
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DRIVERS_SENSOR_INTF_DIGITAL_THERMOSTAT_INC_DS620_INTERFACE_H_
#define DRIVERS_SENSOR_INTF_DIGITAL_THERMOSTAT_INC_DS620_INTERFACE_H_

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include "exo_ahw_al_dt_common.h"
#include "exo_osal_timer.h"
#include "sensor_driver_intf.h"

/**
 * @brief Digital thermostat data unit structure definition
 */
typedef struct
{
	double temperature;				/*!< temperature in celsius				*/
	uint64_t dt_epoch_time;			/*!< RTC time structure variable        */

}__attribute__((packed, aligned(4)))dt_data_s;

/**
 * @brief Application API to read the temperature with given instance id
 * @param dt_inst_id - digital thermostat instance ID
 * @param dt_param - temperature
 * @retval No return
 */
void dt_get_func(double *dt_param,uint8 dt_inst_id);


#endif /* DRIVERS_SENSOR_INTF_DIGITAL_THERMOSTAT_INC_DS620_INTERFACE_H_ */
