/*
 * @file adm1176_interface.h
 *
 * @brief This file contains structures,function prototype to read and
 * store the data from hotswap cotroller
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DRIVERS_SENSOR_INTERFACE_HOT_SWP_CNTLR_INC_ADM1176_INTERFACE_H_
#define DRIVERS_SENSOR_INTERFACE_HOT_SWP_CNTLR_INC_ADM1176_INTERFACE_H_

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include "exo_ahw_al_hsc_common.h"
#include "exo_osal_timer.h"
#include "sensor_driver_intf.h"

/**
 * @brief Hotswap controller data structure definition
 */
typedef struct
{
	float voltage;			/*!< voltage			*/
	float current;			/*!< current			*/

}__attribute__((packed, aligned(4)))adm_get_info_s;

/**
 * @brief Hotswap controller data unit structure definition
 */
typedef struct
{
	adm_get_info_s hsc_data;			/*!< Hotswap controller data structure variable     */
	uint64_t adm_epoch_time;			/*!< RTC time structure variable   					*/
}__attribute__((packed, aligned(4)))hsc_data_s;

/**
 * @brief Application API to read the voltage with the given instance id
 * @param adm_param - pointer to a Hotswap controller information structure
 * @param adm_inst_id - Hotswap controller instance id
 * @retval No return
 */
void hsc_get_func(adm_get_info_s *adm_param,uint8_t adm_inst_id);


#endif /* DRIVERS_SENSOR_INTERFACE_HOT_SWP_CNTLR_INC_ADM1176_INTERFACE_H_ */
