/*
 * @file mcp9843_intf.h
 *
 * @brief This file contains structures,function prototype to read and
 * store the data from temperature sensor
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef DRIVERS_SENSOR_INTF_TEMP_SENSOR_INC_MCP9843_INTF_H_
#define DRIVERS_SENSOR_INTF_TEMP_SENSOR_INC_MCP9843_INTF_H_

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include "exo_ahw_al_temp_sensor_common.h"
#include "exo_osal_timer.h"
#include "sensor_driver_intf.h"

/**
 * @brief Temperature sensor data unit structure
 */
typedef struct
{
	float temperature;				/*!< temperature							*/
	uint64_t temp_epoch_time;		/*!< RTC time structure variable			*/

}__attribute__((packed, aligned(4)))temp_data_s;

/**
 * @brief Application API to get the temperature with the given instance id
 * @param temp_param - temperature in celsius
 * @param temp_inst_id - temperature instance id
 * @retval No return
 */
void temp_get_func(float *temp_param,uint8 temp_inst_id);

#endif /* DRIVERS_SENSOR_INTF_TEMP_SENSOR_INC_MCP9843_INTF_H_ */
