/**
 * @file comms_sband_api_map.h
 *
 * @brief This file contains API mapping of various S-Band module/S_Band simulation
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0(the "License");
 *you may not use this file except in compliance with the License.
 *you may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*
*/

#ifndef COMMS_SBAND_API_MAP_H_
#define COMMS_SBAND_API_MAP_H_

#include<comms_csw.h>
#include<comms_sband_sim.h>
#include "comms_sband_api.h"

#ifndef S_BAND_SIM_CB

/**
* @brief These are Macros used for programming API mapping of SRS4 S-Band module.
*
*/

#define SDR_SBAND_SET_CAL_CFG(x)                    sdr_srs4_sband_set_cal_cfg(x)
#define SDR_SBAND_SET_SYS_CFG(x)                    sdr_srs4_sband_set_sys_cfg(x)
#define SDR_SBAND_GET_SYS_CFG(x)                    sdr_srs4_sband_get_sys_cfg(x)
#define SDR_SBAND_GET_CAL_CFG(x)                    sdr_srs4_sband_get_cal_cfg(x)
#define SDR_SBAND_SET_RX_CFG(x)                     sdr_srs4_sband_set_rx_cfg(x)
#define SDR_SBAND_SET_TX_CFG(x)                     sdr_srs4_sband_set_tx_cfg(x)
#define SDR_SBAND_GET_RX_CFG(x)                     sdr_srs4_sband_get_rx_cfg(x)
#define SDR_SBAND_GET_TX_CFG(x)                     sdr_srs4_sband_get_tx_cfg(x)
#define SDR_SBAND_FIRMWARE_CFG(x)                   sdr_srs4_sband_firmware_cfg(x)
#define SDR_SBAND_SET_CFG(x,y,z)                    sdr_srs4_sband_set_cfg(x,y,  z)
#define SDR_SBAND_GET_CFG(x,y,z)                    sdr_srs4_sband_get_cfg(x,y,  z)
#define SDR_SBAND_RESET_REQ(x)                      sdr_srs4_sband_reset_req(x)
#define SDR_SBAND_GROUP_SAVE_REQ(x,y)               sdr_srs4_sband_group_save_req(x,y)
#define SDR_SBAND_GROUP_LOAD_REQ(x,y)               sdr_srs4_sband_group_load_req(x,y)
#define SDR_SBAND_GROUP_ERASE_REQ(x,y)              sdr_srs4_sband_group_erase_req(x,y)
#define SDR_SBAND_GROUP_LOCK_REQ(x,y)               sdr_srs4_sband_group_lock_req(x,y)
#define SDR_SBAND_GROUP_UNLOCK_REQ(x,y)             sdr_srs4_sband_group_unlock_req(x,y)
#define SDR_SBAND_SET_TX_CFG_FREQ(x)                sdr_srs4_sband_set_tx_cfg_freq(x)
#define SDR_SBAND_SET_TX_CFG_TYPE(x)                sdr_srs4_sband_set_tx_cfg_type(x)
#define SDR_SBAND_SET_TX_CFG_RATE(x)                sdr_srs4_sband_set_tx_cfg_rate(x)
#define SDR_SBAND_SET_TX_CFG_ROLLOFF(x)             sdr_srs4_sband_set_tx_cfg_rolloff(x)
#define SDR_SBAND_SET_TX_CFG_POUT(x)                sdr_srs4_sband_set_tx_cfg_pout(x)
#define SDR_SBAND_SET_TX_CFG_ALC_MODE(x)            sdr_srs4_sband_set_tx_cfg_alc_mode(x)
#define SDR_SBAND_SET_TX_CFG_RS(x)                  sdr_srs4_sband_set_tx_cfg_rs(x)
#define SDR_SBAND_SET_TX_CFG_CC(x)                  sdr_srs4_sband_set_tx_cfg_cc(x)
#define SDR_SBAND_SET_TX_CFG_RAND(x)                sdr_srs4_sband_set_tx_cfg_rand(x)
#define SDR_SBAND_SET_TX_CFG_CRC(x)                 sdr_srs4_sband_set_tx_cfg_crc(x)
#define SDR_SBAND_SET_TX_CFG_IDLETIME(x)            sdr_srs4_sband_set_tx_cfg_idletime(x)
#define SDR_SBAND_SET_TX_CFG_SIZE(x)                sdr_srs4_sband_set_tx_cfg_size(x)
#define SDR_SBAND_SET_TX_CFG_ID(x)                  sdr_srs4_sband_set_tx_cfg_id(x)
#define SDR_SBAND_SET_TX_CFG_CRYPTO(x)              sdr_srs4_sband_set_tx_cfg_crypto(x)
#define SDR_SBAND_SET_TX_CFG_PA_TEMPLIMIT(x)        sdr_srs4_sband_set_tx_cfg_pa_templimit(x)
#define SDR_SBAND_GET_TX_CFG_FREQ(x,y)              sdr_srs4_sband_get_tx_cfg_freq(x,y)
#define SDR_SBAND_GET_TX_CFG_TYPE(x,y)              sdr_srs4_sband_get_tx_cfg_type(x,y)
#define SDR_SBAND_GET_TX_CFG_RATE(x,y)              sdr_srs4_sband_get_tx_cfg_rate(x,y)
#define SDR_SBAND_GET_TX_CFG_ROLLOFF(x,y)           sdr_srs4_sband_get_tx_cfg_rolloff(x,y)
#define SDR_SBAND_GET_TX_CFG_POUT(x,y)              sdr_srs4_sband_get_tx_cfg_pout(x,y)
#define SDR_SBAND_GET_TX_CFG_ALC_MODE(x,y)          sdr_srs4_sband_get_tx_cfg_alc_mode(x,y)
#define SDR_SBAND_GET_TX_CFG_RS(x,y)                sdr_srs4_sband_get_tx_cfg_rs(x,y)
#define SDR_SBAND_GET_TX_CFG_CC(x,y)                sdr_srs4_sband_get_tx_cfg_cc(x,y)
#define SDR_SBAND_GET_TX_CFG_RAND(x,y)              sdr_srs4_sband_get_tx_cfg_rand(x,y)
#define SDR_SBAND_GET_TX_CFG_CRC(x,y)               sdr_srs4_sband_get_tx_cfg_crc(x,y)
#define SDR_SBAND_GET_TX_CFG_IDLETIME(x,y)          sdr_srs4_sband_get_tx_cfg_idletime(x,y)
#define SDR_SBAND_GET_TX_CFG_SIZE(x,y)              sdr_srs4_sband_get_tx_cfg_size(x,y)
#define SDR_SBAND_GET_TX_CFG_ID(x,y)                sdr_srs4_sband_get_tx_cfg_id(x,y)
#define SDR_SBAND_GET_TX_CFG_CRYPTO(x,y)            sdr_srs4_sband_get_tx_cfg_crypto(x,y)
#define SDR_SBAND_GET_TX_CFG_PA_TEMPLIMIT(x,y)      sdr_srs4_sband_get_tx_cfg_pa_templimit(x,y)
#define SDR_SBAND_GET_TX_CFG_FRAMES(x,y)            sdr_srs4_sband_get_tx_cfg_frames(x,y)
#define SDR_SBAND_GET_TX_CFG_PWR_FWD(x,y)           sdr_srs4_sband_get_tx_cfg_pwr_fwd(x,y)
#define SDR_SBAND_GET_TX_CFG_PWR_RFL(x,y)           sdr_srs4_sband_get_tx_cfg_pwr_rfl(x,y)
#define SDR_SBAND_SET_RX_CFG_FREQ(x)                sdr_srs4_sband_set_rx_cfg_freq(x)
#define SDR_SBAND_SET_RX_CFG_TYPE(x)                sdr_srs4_sband_set_rx_cfg_type(x)
#define SDR_SBAND_SET_RX_CFG_RATE(x)                sdr_srs4_sband_set_rx_cfg_rate(x)
#define SDR_SBAND_SET_RX_CFG_ROLLOFF(x)             sdr_srs4_sband_set_rx_cfg_rolloff(x)
#define SDR_SBAND_SET_RX_CFG_AFC(x)                 sdr_srs4_sband_set_rx_cfg_afc(x)
#define SDR_SBAND_SET_RX_CFG_RS(x)                  sdr_srs4_sband_set_rx_cfg_rs(x)
#define SDR_SBAND_SET_RX_CFG_CC(x)                  sdr_srs4_sband_set_rx_cfg_cc(x)
#define SDR_SBAND_SET_RX_CFG_RAND(x)                sdr_srs4_sband_set_rx_cfg_rand(x)
#define SDR_SBAND_SET_RX_CFG_CRC(x)                 sdr_srs4_sband_set_rx_cfg_crc(x)
#define SDR_SBAND_SET_RX_CFG_SIZE(x)                sdr_srs4_sband_set_rx_cfg_size(x)
#define SDR_SBAND_SET_RX_CFG_ID(x)                  sdr_srs4_sband_set_rx_cfg_id(x)
#define SDR_SBAND_SET_RX_CFG_CRYPTO(x)              sdr_srs4_sband_set_rx_cfg_crypto(x)
#define SDR_SBAND_GET_RX_CFG_FREQ(x,y)              sdr_srs4_sband_get_rx_cfg_freq(x,y)
#define SDR_SBAND_GET_RX_CFG_TYPE(x,y)              sdr_srs4_sband_get_rx_cfg_type(x,y)
#define SDR_SBAND_GET_RX_CFG_RATE(x,y)              sdr_srs4_sband_get_rx_cfg_rate(x,y)
#define SDR_SBAND_GET_RX_CFG_ROLLOFF(x,y)           sdr_srs4_sband_get_rx_cfg_rolloff(x,y)
#define SDR_SBAND_GET_RX_CFG_AFC(x,y)               sdr_srs4_sband_get_rx_cfg_afc(x,y)
#define SDR_SBAND_GET_RX_CFG_RS(x,y)                sdr_srs4_sband_get_rx_cfg_rs(x,y)
#define SDR_SBAND_GET_RX_CFG_CC(x,y)                sdr_srs4_sband_get_rx_cfg_cc(x,y)
#define SDR_SBAND_GET_RX_CFG_RAND(x,y)              sdr_srs4_sband_get_rx_cfg_rand(x,y)
#define SDR_SBAND_GET_RX_CFG_CRC(x,y)               sdr_srs4_sband_get_rx_cfg_crc(x,y)
#define SDR_SBAND_GET_RX_CFG_SIZE(x,y)              sdr_srs4_sband_get_rx_cfg_size(x,y)
#define SDR_SBAND_GET_RX_CFG_ID(x,y)                sdr_srs4_sband_get_rx_cfg_id(x,y)
#define SDR_SBAND_GET_RX_CFG_CRYPTO(x,y)            sdr_srs4_sband_get_rx_cfg_crypto(x,y)
#define SDR_SBAND_GET_RX_CFG_FRAMES(x,y)            sdr_srs4_sband_get_rx_cfg_frames(x,y)
#define SDR_SBAND_GET_RX_CFG_DETECTED(x,y)          sdr_srs4_sband_get_rx_cfg_detected(x,y)
#define SDR_SBAND_GET_RX_CFG_RSSI(x,y)              sdr_srs4_sband_get_rx_cfg_rssi(x,y)
#define SDR_SBAND_GET_RX_CFG_FREQERR(x,y)           sdr_srs4_sband_get_rx_cfg_freqerr(x,y)
#define SDR_SBAND_GET_RX_CFG_SYMERR(x,y)            sdr_srs4_sband_get_rx_cfg_symerr(x,y)
#define SDR_SBAND_GET_RX_CFG_EBN0(x,y)              sdr_srs4_sband_get_rx_cfg_ebn0(x,y)
#define SDR_SBAND_GET_RX_CFG_CARRIER_LOCK(x,y)      sdr_srs4_sband_get_rx_cfg_carrier_lock(x,y)
#define SDR_SBAND_GET_RX_CFG_FRAME_LOCK(x,y)        sdr_srs4_sband_get_rx_cfg_frame_lock(x,y)
#define SDR_SBAND_SET_SYS_CFG_NAME(x)               sdr_srs4_sband_set_sys_cfg_name(x)
#define SDR_SBAND_SET_SYS_CFG_BOOT_CFG(x)           sdr_srs4_sband_set_sys_cfg_boot_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_GWDT_CFG(x)           sdr_srs4_sband_set_sys_cfg_gwdt_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_CSP_CFG(x)            sdr_srs4_sband_set_sys_cfg_csp_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_CAN_CFG(x)            sdr_srs4_sband_set_sys_cfg_can_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_UART_CFG(x)           sdr_srs4_sband_set_sys_cfg_uart_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_NET_CFG(x)            sdr_srs4_sband_set_sys_cfg_net_cfg(x)
#define SDR_SBAND_GET_SYS_CFG_NAME(x)               sdr_srs4_sband_get_sys_cfg_name(x)
#define SDR_SBAND_GET_SYS_CFG_BOOT_CFG(x,y)         sdr_srs4_sband_get_sys_cfg_boot_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_GWDT_CFG(x,y)         sdr_srs4_sband_get_sys_cfg_gwdt_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_CSP_CFG(x,y)          sdr_srs4_sband_get_sys_cfg_csp_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_CAN_CFG(x,y)          sdr_srs4_sband_get_sys_cfg_can_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_UART_CFG(x,y)         sdr_srs4_sband_get_sys_cfg_uart_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_NET_CFG(x,y)          sdr_srs4_sband_get_sys_cfg_net_cfg(x,y)
#define SDR_SBAND_SET_CAL_CFG_PS_GAIN(x)            sdr_srs4_sband_set_cal_cfg_ps_gain(x)
#define SDR_SBAND_SET_CAL_CFG_PS_INTERCEPT(x)       sdr_srs4_sband_set_cal_cfg_ps_intercept(x)
#define SDR_SBAND_SET_CAL_CFG_VCTCXO(x)             sdr_srs4_sband_set_cal_cfg_vctcxo(x)
#define SDR_SBAND_GET_CAL_CFG_PS_GAIN(x,y)          sdr_srs4_sband_get_cal_cfg_ps_gain(x,y)
#define SDR_SBAND_GET_CAL_CFG_PS_INTERCEPT(x,y)     sdr_srs4_sband_get_cal_cfg_ps_intercept(x,y)
#define SDR_SBAND_GET_CAL_CFG_VCTCXO(x,y)           sdr_srs4_sband_get_cal_cfg_vctcxo(x,y)
#define SDR_SBAND_GET_CAL_CFG_PWR_SET(x,y)          sdr_srs4_sband_get_cal_cfg_pwr_set(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_ENABLE(x,y)       sdr_srs4_sband_get_cal_cfg_ber_enable(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_WINDOW(x,y)       sdr_srs4_sband_get_cal_cfg_ber_window(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_COUNT(x,y)        sdr_srs4_sband_get_cal_cfg_ber_count(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_LOCKS(x,y)        sdr_srs4_sband_get_cal_cfg_ber_locks(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_PSU(x,y)         sdr_srs4_sband_get_tm_prop_temp_psu(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_MCU(x,y)         sdr_srs4_sband_get_tm_prop_temp_mcu(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_FPGA(x,y)        sdr_srs4_sband_get_tm_prop_temp_fpga(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_XCVR(x,y)        sdr_srs4_sband_get_tm_prop_temp_xcvr(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_LNA(x,y)         sdr_srs4_sband_get_tm_prop_temp_lna(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_PA(x,y)          sdr_srs4_sband_get_tm_prop_temp_pa(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_VIN(x,y)         sdr_srs4_sband_get_tm_prop_volt_vin(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_3V3(x,y)         sdr_srs4_sband_get_tm_prop_volt_3v3(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_1V8(x,y)         sdr_srs4_sband_get_tm_prop_volt_1v8(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_1V0(x,y)         sdr_srs4_sband_get_tm_prop_volt_1v0(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_PA(x,y)          sdr_srs4_sband_get_tm_prop_volt_pa(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_VIN(x,y)          sdr_srs4_sband_get_tm_prop_cur_vin(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_3V3(x,y)          sdr_srs4_sband_get_tm_prop_cur_3v3(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_1V8(x,y)          sdr_srs4_sband_get_tm_prop_cur_1v8(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_1V0(x,y)          sdr_srs4_sband_get_tm_prop_cur_1v0(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_PA(x,y)           sdr_srs4_sband_get_tm_prop_cur_pa(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_VIN(x,y)        sdr_srs4_sband_get_tm_prop_power_vin(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_3V3(x,y)        sdr_srs4_sband_get_tm_prop_power_3v3(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_1V8(x,y)        sdr_srs4_sband_get_tm_prop_power_1v8(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_1V0(x,y)        sdr_srs4_sband_get_tm_prop_power_1v0(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_PA(x,y)         sdr_srs4_sband_get_tm_prop_power_pa(x,y)
#define SDR_SBAND_GET_TX_TM_PROP(x,y)               sdr_srs4_sband_get_tx_tm_prop(x,y)
#define SDR_SBAND_GET_RX_TM_PROP(x,y)               sdr_srs4_sband_get_rx_tm_prop(x,y)
#define SDR_SBAND_GET_TM_PROP(x,y)                  sdr_srs4_sband_get_tm_prop(x,y)
#define SDR_SBAND_GET_CSP_TM_PROP(x,y)              sdr_get_csp_tm_prop(x,y)

#else

/**
* @brief These are Macros used for programming API mapping of S-Band simulation module.
*
*/

#define SDR_SBAND_SET_CAL_CFG(x)                    sdr_sband_sim_set_cal_cfg(x)
#define SDR_SBAND_SET_SYS_CFG(x)                    sdr_sband_sim_set_sys_cfg(x)
#define SDR_SBAND_GET_SYS_CFG(x)                    sdr_sband_sim_get_sys_cfg(x)
#define SDR_SBAND_GET_CAL_CFG(x)                    sdr_sband_sim_get_cal_cfg(x)
#define SDR_SBAND_SET_RX_CFG(x)                     sdr_sband_sim_set_rx_cfg(x)
#define SDR_SBAND_SET_TX_CFG(x)                     sdr_sband_sim_set_tx_cfg(x)
#define SDR_SBAND_GET_RX_CFG(x)                     sdr_sband_sim_get_rx_cfg(x)
#define SDR_SBAND_GET_TX_CFG(x)                     sdr_sband_sim_get_tx_cfg(x)
#define SDR_SBAND_FIRMWARE_CFG(x)                   sdr_sband_sim_firmware_cfg(x)
#define SDR_SBAND_SET_CFG(x,y,z)                    sdr_sband_sim_set_cfg(x,y,  z)
#define SDR_SBAND_GET_CFG(x,y,z)                    sdr_sband_sim_get_cfg(x,y,  z)
#define SDR_SBAND_RESET_REQ(x)                      sdr_sband_sim_reset_req(x)
#define SDR_SBAND_GROUP_SAVE_REQ(x,y)               sdr_sband_sim_group_save_req(x,y)
#define SDR_SBAND_GROUP_LOAD_REQ(x,y)               sdr_sband_sim_group_load_req(x,y)
#define SDR_SBAND_GROUP_ERASE_REQ(x,y)              sdr_sband_sim_group_erase_req(x,y)
#define SDR_SBAND_GROUP_LOCK_REQ(x,y)               sdr_sband_sim_group_lock_req(x,y)
#define SDR_SBAND_GROUP_UNLOCK_REQ(x,y)             sdr_sband_sim_group_unlock_req(x,y)
#define SDR_SBAND_SET_TX_CFG_FREQ(x)                sdr_sband_sim_set_tx_cfg_freq(x)
#define SDR_SBAND_SET_TX_CFG_TYPE(x)                sdr_sband_sim_set_tx_cfg_type(x)
#define SDR_SBAND_SET_TX_CFG_RATE(x)                sdr_sband_sim_set_tx_cfg_rate(x)
#define SDR_SBAND_SET_TX_CFG_ROLLOFF(x)             sdr_sband_sim_set_tx_cfg_rolloff(x)
#define SDR_SBAND_SET_TX_CFG_POUT(x)                sdr_sband_sim_set_tx_cfg_pout(x)
#define SDR_SBAND_SET_TX_CFG_ALC_MODE(x)            sdr_sband_sim_set_tx_cfg_alc_mode(x)
#define SDR_SBAND_SET_TX_CFG_RS(x)                  sdr_sband_sim_set_tx_cfg_rs(x)
#define SDR_SBAND_SET_TX_CFG_CC(x)                  sdr_sband_sim_set_tx_cfg_cc(x)
#define SDR_SBAND_SET_TX_CFG_RAND(x)                sdr_sband_sim_set_tx_cfg_rand(x)
#define SDR_SBAND_SET_TX_CFG_CRC(x)                 sdr_sband_sim_set_tx_cfg_crc(x)
#define SDR_SBAND_SET_TX_CFG_IDLETIME(x)            sdr_sband_sim_set_tx_cfg_idletime(x)
#define SDR_SBAND_SET_TX_CFG_SIZE(x)                sdr_sband_sim_set_tx_cfg_size(x)
#define SDR_SBAND_SET_TX_CFG_ID(x)                  sdr_sband_sim_set_tx_cfg_id(x)
#define SDR_SBAND_SET_TX_CFG_CRYPTO(x)              sdr_sband_sim_set_tx_cfg_crypto(x)
#define SDR_SBAND_SET_TX_CFG_PA_TEMPLIMIT(x)        sdr_sband_sim_set_tx_cfg_pa_templimit(x)
#define SDR_SBAND_GET_TX_CFG_FREQ(x,y)              sdr_sband_sim_get_tx_cfg_freq(x,y)
#define SDR_SBAND_GET_TX_CFG_TYPE(x,y)              sdr_sband_sim_get_tx_cfg_type(x,y)
#define SDR_SBAND_GET_TX_CFG_RATE(x,y)              sdr_sband_sim_get_tx_cfg_rate(x,y)
#define SDR_SBAND_GET_TX_CFG_ROLLOFF(x,y)           sdr_sband_sim_get_tx_cfg_rolloff(x,y)
#define SDR_SBAND_GET_TX_CFG_POUT(x,y)              sdr_sband_sim_get_tx_cfg_pout(x,y)
#define SDR_SBAND_GET_TX_CFG_ALC_MODE(x,y)          sdr_sband_sim_get_tx_cfg_alc_mode(x,y)
#define SDR_SBAND_GET_TX_CFG_RS(x,y)                sdr_sband_sim_get_tx_cfg_rs(x,y)
#define SDR_SBAND_GET_TX_CFG_CC(x,y)                sdr_sband_sim_get_tx_cfg_cc(x,y)
#define SDR_SBAND_GET_TX_CFG_RAND(x,y)              sdr_sband_sim_get_tx_cfg_rand(x,y)
#define SDR_SBAND_GET_TX_CFG_CRC(x,y)               sdr_sband_sim_get_tx_cfg_crc(x,y)
#define SDR_SBAND_GET_TX_CFG_IDLETIME(x,y)          sdr_sband_sim_get_tx_cfg_idletime(x,y)
#define SDR_SBAND_GET_TX_CFG_SIZE(x,y)              sdr_sband_sim_get_tx_cfg_size(x,y)
#define SDR_SBAND_GET_TX_CFG_ID(x,y)                sdr_sband_sim_get_tx_cfg_id(x,y)
#define SDR_SBAND_GET_TX_CFG_CRYPTO(x,y)            sdr_sband_sim_get_tx_cfg_crypto(x,y)
#define SDR_SBAND_GET_TX_CFG_PA_TEMPLIMIT(x,y)      sdr_sband_sim_get_tx_cfg_pa_templimit(x,y)
#define SDR_SBAND_GET_TX_CFG_FRAMES(x,y)            sdr_sband_sim_get_tx_cfg_frames(x,y)
#define SDR_SBAND_GET_TX_CFG_PWR_FWD(x,y)           sdr_sband_sim_get_tx_cfg_pwr_fwd(x,y)
#define SDR_SBAND_GET_TX_CFG_PWR_RFL(x,y)           sdr_sband_sim_get_tx_cfg_pwr_rfl(x,y)
#define SDR_SBAND_SET_RX_CFG_FREQ(x)                sdr_sband_sim_set_rx_cfg_freq(x)
#define SDR_SBAND_SET_RX_CFG_TYPE(x)                sdr_sband_sim_set_rx_cfg_type(x)
#define SDR_SBAND_SET_RX_CFG_RATE(x)                sdr_sband_sim_set_rx_cfg_rate(x)
#define SDR_SBAND_SET_RX_CFG_ROLLOFF(x)             sdr_sband_sim_set_rx_cfg_rolloff(x)
#define SDR_SBAND_SET_RX_CFG_AFC(x)                 sdr_sband_sim_set_rx_cfg_afc(x)
#define SDR_SBAND_SET_RX_CFG_RS(x)                  sdr_sband_sim_set_rx_cfg_rs(x)
#define SDR_SBAND_SET_RX_CFG_CC(x)                  sdr_sband_sim_set_rx_cfg_cc(x)
#define SDR_SBAND_SET_RX_CFG_RAND(x)                sdr_sband_sim_set_rx_cfg_rand(x)
#define SDR_SBAND_SET_RX_CFG_CRC(x)                 sdr_sband_sim_set_rx_cfg_crc(x)
#define SDR_SBAND_SET_RX_CFG_SIZE(x)                sdr_sband_sim_set_rx_cfg_size(x)
#define SDR_SBAND_SET_RX_CFG_ID(x)                  sdr_sband_sim_set_rx_cfg_id(x)
#define SDR_SBAND_SET_RX_CFG_CRYPTO(x)              sdr_sband_sim_set_rx_cfg_crypto(x)
#define SDR_SBAND_GET_RX_CFG_FREQ(x,y)              sdr_sband_sim_get_rx_cfg_freq(x,y)
#define SDR_SBAND_GET_RX_CFG_TYPE(x,y)              sdr_sband_sim_get_rx_cfg_type(x,y)
#define SDR_SBAND_GET_RX_CFG_RATE(x,y)              sdr_sband_sim_get_rx_cfg_rate(x,y)
#define SDR_SBAND_GET_RX_CFG_ROLLOFF(x,y)           sdr_sband_sim_get_rx_cfg_rolloff(x,y)
#define SDR_SBAND_GET_RX_CFG_AFC(x,y)               sdr_sband_sim_get_rx_cfg_afc(x,y)
#define SDR_SBAND_GET_RX_CFG_RS(x,y)                sdr_sband_sim_get_rx_cfg_rs(x,y)
#define SDR_SBAND_GET_RX_CFG_CC(x,y)                sdr_sband_sim_get_rx_cfg_cc(x,y)
#define SDR_SBAND_GET_RX_CFG_RAND(x,y)              sdr_sband_sim_get_rx_cfg_rand(x,y)
#define SDR_SBAND_GET_RX_CFG_CRC(x,y)               sdr_sband_sim_get_rx_cfg_crc(x,y)
#define SDR_SBAND_GET_RX_CFG_SIZE(x,y)              sdr_sband_sim_get_rx_cfg_size(x,y)
#define SDR_SBAND_GET_RX_CFG_ID(x,y)                sdr_sband_sim_get_rx_cfg_id(x,y)
#define SDR_SBAND_GET_RX_CFG_CRYPTO(x,y)            sdr_sband_sim_get_rx_cfg_crypto(x,y)
#define SDR_SBAND_GET_RX_CFG_FRAMES(x,y)            sdr_sband_sim_get_rx_cfg_frames(x,y)
#define SDR_SBAND_GET_RX_CFG_DETECTED(x,y)          sdr_sband_sim_get_rx_cfg_detected(x,y)
#define SDR_SBAND_GET_RX_CFG_RSSI(x,y)              sdr_sband_sim_get_rx_cfg_rssi(x,y)
#define SDR_SBAND_GET_RX_CFG_FREQERR(x,y)           sdr_sband_sim_get_rx_cfg_freqerr(x,y)
#define SDR_SBAND_GET_RX_CFG_SYMERR(x,y)            sdr_sband_sim_get_rx_cfg_symerr(x,y)
#define SDR_SBAND_GET_RX_CFG_EBN0(x,y)              sdr_sband_sim_get_rx_cfg_ebn0(x,y)
#define SDR_SBAND_GET_RX_CFG_CARRIER_LOCK(x,y)      sdr_sband_sim_get_rx_cfg_carrier_lock(x,y)
#define SDR_SBAND_GET_RX_CFG_FRAME_LOCK(x,y)        sdr_sband_sim_get_rx_cfg_frame_lock(x,y)
#define SDR_SBAND_SET_SYS_CFG_NAME(x)               sdr_sband_sim_set_sys_cfg_name(x)
#define SDR_SBAND_SET_SYS_CFG_BOOT_CFG(x)           sdr_sband_sim_set_sys_cfg_boot_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_GWDT_CFG(x)           sdr_sband_sim_set_sys_cfg_gwdt_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_CSP_CFG(x)            sdr_sband_sim_set_sys_cfg_csp_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_CAN_CFG(x)            sdr_sband_sim_set_sys_cfg_can_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_UART_CFG(x)           sdr_sband_sim_set_sys_cfg_uart_cfg(x)
#define SDR_SBAND_SET_SYS_CFG_NET_CFG(x)            sdr_sband_sim_set_sys_cfg_net_cfg(x)
#define SDR_SBAND_GET_SYS_CFG_NAME(x)               sdr_sband_sim_get_sys_cfg_name(x)
#define SDR_SBAND_GET_SYS_CFG_BOOT_CFG(x,y)         sdr_sband_sim_get_sys_cfg_boot_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_GWDT_CFG(x,y)         sdr_sband_sim_get_sys_cfg_gwdt_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_CSP_CFG(x,y)          sdr_sband_sim_get_sys_cfg_csp_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_CAN_CFG(x,y)          sdr_sband_sim_get_sys_cfg_can_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_UART_CFG(x,y)         sdr_sband_sim_get_sys_cfg_uart_cfg(x,y)
#define SDR_SBAND_GET_SYS_CFG_NET_CFG(x,y)          sdr_sband_sim_get_sys_cfg_net_cfg(x,y)
#define SDR_SBAND_SET_CAL_CFG_PS_GAIN(x)            sdr_sband_sim_set_cal_cfg_ps_gain(x)
#define SDR_SBAND_SET_CAL_CFG_PS_INTERCEPT(x)       sdr_sband_sim_set_cal_cfg_ps_intercept(x)
#define SDR_SBAND_SET_CAL_CFG_VCTCXO(x)             sdr_sband_sim_set_cal_cfg_vctcxo(x)
#define SDR_SBAND_GET_CAL_CFG_PS_GAIN(x,y)          sdr_sband_sim_get_cal_cfg_ps_gain(x,y)
#define SDR_SBAND_GET_CAL_CFG_PS_INTERCEPT(x,y)     sdr_sband_sim_get_cal_cfg_ps_intercept(x,y)
#define SDR_SBAND_GET_CAL_CFG_VCTCXO(x,y)           sdr_sband_sim_get_cal_cfg_vctcxo(x,y)
#define SDR_SBAND_GET_CAL_CFG_PWR_SET(x,y)          sdr_sband_sim_get_cal_cfg_pwr_set(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_ENABLE(x,y)       sdr_sband_sim_get_cal_cfg_ber_enable(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_WINDOW(x,y)       sdr_sband_sim_get_cal_cfg_ber_window(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_COUNT(x,y)        sdr_sband_sim_get_cal_cfg_ber_count(x,y)
#define SDR_SBAND_GET_CAL_CFG_BER_LOCKS(x,y)        sdr_sband_sim_get_cal_cfg_ber_locks(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_PSU(x,y)         sdr_sband_sim_get_tm_prop_temp_psu(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_MCU(x,y)         sdr_sband_sim_get_tm_prop_temp_mcu(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_FPGA(x,y)        sdr_sband_sim_get_tm_prop_temp_fpga(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_XCVR(x,y)        sdr_sband_sim_get_tm_prop_temp_xcvr(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_LNA(x,y)         sdr_sband_sim_get_tm_prop_temp_lna(x,y)
#define SDR_SBAND_GET_TM_PROP_TEMP_PA(x,y)          sdr_sband_sim_get_tm_prop_temp_pa(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_VIN(x,y)         sdr_sband_sim_get_tm_prop_volt_vin(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_3V3(x,y)         sdr_sband_sim_get_tm_prop_volt_3v3(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_1V8(x,y)         sdr_sband_sim_get_tm_prop_volt_1v8(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_1V0(x,y)         sdr_sband_sim_get_tm_prop_volt_1v0(x,y)
#define SDR_SBAND_GET_TM_PROP_VOLT_PA(x,y)          sdr_sband_sim_get_tm_prop_volt_pa(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_VIN(x,y)          sdr_sband_sim_get_tm_prop_cur_vin(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_3V3(x,y)          sdr_sband_sim_get_tm_prop_cur_3v3(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_1V8(x,y)          sdr_sband_sim_get_tm_prop_cur_1v8(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_1V0(x,y)          sdr_sband_sim_get_tm_prop_cur_1v0(x,y)
#define SDR_SBAND_GET_TM_PROP_CUR_PA(x,y)           sdr_sband_sim_get_tm_prop_cur_pa(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_VIN(x,y)        sdr_sband_sim_get_tm_prop_power_vin(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_3V3(x,y)        sdr_sband_sim_get_tm_prop_power_3v3(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_1V8(x,y)        sdr_sband_sim_get_tm_prop_power_1v8(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_1V0(x,y)        sdr_sband_sim_get_tm_prop_power_1v0(x,y)
#define SDR_SBAND_GET_TM_PROP_POWER_PA(x,y)         sdr_sband_sim_get_tm_prop_power_pa(x,y)
#define SDR_SBAND_GET_TX_TM_PROP(x,y)               sdr_sband_sim_get_tx_tm_prop(x,y)
#define SDR_SBAND_GET_RX_TM_PROP(x,y)               sdr_sband_sim_get_rx_tm_prop(x,y)
#define SDR_SBAND_GET_TM_PROP(x,y)                  sdr_sband_sim_get_tm_prop(x,y)
#define SDR_SBAND_GET_CSP_TM_PROP(x,y)              sdr_get_csp_tm_prop(x,y)

#endif

#endif /*COMMS_SBAND_API_MAP_H_*/
