/**
 * @file ft_receiver.h
 *
 * @brief This file contains function prototypes and data definitions of state machine APIs for receiver side
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef _FT_RECEIVER_H_
#define _FT_RECEIVER_H_

#include "stdint.h"
#include "tc_tm_sample.h"



/******************************************************************************/
/*               Declaration of exported function prototypes                  */
/******************************************************************************/

/**
 * @brief State Machine :Function that handles download operation
 *
 *
 * @param[in]  none
 *
 * @return none
 * @retval void
 */
void ft_statemachine_download(void);

/**
 * @brief This function is used to clear the ping failure monitor counter
 *
 *
 * @param[in]  none
 *
 * @return none
 * @retval void
 */
void ft_receiver_ping_rest(void);

/**
 * @brief This function is used to set the app id of receiver side
 *
 *
 * @param[in]  set : app id to set
 *
 * @return none
 * @retval void
 */
void ft_set_app_id_rx(uint16_t set);

/**
 * @brief This function is used to get the app id of receiver side
 *
 *
 * @param[in]  void : none
 *
 * @return app id
 * @retval void
 */
uint16_t ft_get_app_id_rx(void);
void ft_rx_session_id_store(uint8 sid);
uint8_t ft_rx_session_id_read(void);


#endif /* _FT_RECEIVER_H_ */
