/**
 *@file ftds_decode.h
 *
 *@brief This file contains function prototypes and data definitions that
 *helps to decode FTDS frame and storing data
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FTDP_DECODE_H_
#define FTDP_DECODE_H_

#include "data_type.h"
#include "notify_apps.h"


/******************************************************************************/
/*               Declaration of exported function prototypes                  */
/******************************************************************************/

/**
 * @brief This API receives the FTDS frame data from TC-TM then decode FTDS frame
 *
 *
 * @param[in]  ftds_data  : pointer to FTDS received frame
 * @param[in]  ftds_data_length  : FTDS data buffer length
 *
 * @return none
 * @retval void
 */
void ft_receive_ftds(uint8_t *ftds_data, uint16_t ftds_data_length);






/**
 * @brief This function return the head sequence ID for receiver side
 *
 *
 * @param[in]  void  : none
 *
 * @return head sequence ID
 * @retval sequence id
 */
uint32_t ft_get_receiver_head_seq_id(void);

/**
 * @brief This function is used to check the sequence loss count status
 *
 *
 * @param[in]  void  : none
 *
 * @return none
 * @retval void
 */
uint8_t ft_get_seq_loss_sts(void);

void ft_seq_loss_handle_init(void);




#endif /*FTDP_DECODE_H_ */
