/**
 * @file ftfci.h
 *
 * @brief This file contains function prototypes and data definitions that
 * helps to handle FTFCI frame encoding
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _APP_INC_FTFCI_H_
#define _APP_INC_FTFCI_H_


#include "message_frame_def.h"
#include "data_type.h"

/******************************************************************************/
/*               Declaration of exported function prototypes                  */
/******************************************************************************/

/**
 * @brief This function used to encode response frame FTFCI
 * It will form FTFCI frame with response code
 *
 * @param[in]  set  : reponse code to be sent
 * @param[in]  waiting_time  : timeout_time value to be shared if sender request for suspend mode
 *
 * @return none
 * @retval void
 */
void send_ftfci(e_ftfci_response_code set, uint16_t waiting_time);



#endif /* FILE_TRANS_REL_V1_SAMPLE_APPS_GS_SAMPLE_APP_INC_FTFCI_H_ */
