/**
 * @file file_transfer.h
 *
 * @brief This file contains function prototypes and data definitions that
 * helps to handle FTFCI decoding operation
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _INC_FTFCI_DECODE_H_
#define _INC_FTFCI_DECODE_H_

#include "file_transfer.h"
#include "message_frame_def.h"

/******************************************************************************/
/*               Declaration of exported function prototypes                  */
/******************************************************************************/

/**
 * @brief This API receives the FTFCI frame data from TC-TM then decode FTFCI frame
 *
 *
 * @param[in]  ftsr_data  : pointer to FTSR received frame
 * @param[in]  ftsr_data_length  : FTSR data buffer length
 *
 * @return none
 * @retval void
 */
void ft_receive_ftfci(uint8_t *ftfci_data, uint16_t ftfci_data_length);



/**
 * @brief This function is used to clear the ping request counter value whenever FTDS has received
 * Receiver will receive ping request for every 20 sequences if sender continuously send
 * only ping for three times, receiver node will ask sender to change the head sequence
 * So the counter will monitor whether the sender only sending consequently ping request
 * for three times.
 *
 *
 * @param[in]  ftsr_data  : pointer to FTSR received frame
 * @param[in]  ftsr_data_length  : FTSR data buffer length
 *
 * @return none
 * @retval void
 */
void ftfci_clear_continous_ping_cntr(void);




#endif /* FILE_TRANS_REL_V1_SAMPLE_APPS_SAT_SAMPLE_APP_INC_FTFCI_DECODE_H_ */
