/**
 * @file ftsr_decode.h
 *
 * @brief This file contains function prototypes and data definitions that
 * helps to decode FTSR frame received from receiver node
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef _FTSR_DECODE_H_
#define _FTSR_DECODE_H_
#include "data_type.h"
#include "message_frame_def.h"
#include "tc_tm_sample.h"
#include "notify_apps.h"



/******************************************************************************/
/*         Definition of exported types (typedef, enum, struct, union)        */
/******************************************************************************/


/**
 * @brief FTSR frame payload info
 */
typedef struct ftsr_frame_buff
{
	uint8_t * ftsr_buf_ptr;  ///< pointer to FSR buffer
	uint16_t buf_length;     ///< buffer length
}
s_ftsr_frame_buff;

/******************************************************************************/
/*               Declaration of exported function prototypes                  */
/******************************************************************************/


/**
 * @brief This API receives the FTSR frame data from TC-TM then decode FTSR frame
 *
 *
 * @param[in]  ftsr_data  : pointer to FTSR received frame
 * @param[in]  ftsr_data_length  : FTSR data buffer length
 *
 * @return none
 * @retval void
 */
void ft_receive_ftsr(uint8_t *ftsr_data, uint16_t ftsr_data_length);





#endif /* FTSR_H_ */
