/**
 * @file tc_tm_sample.h
 *
 * @brief This file contains function prototypes, structure, enumeration 
 *        for file transfer telecommand and telemetry.
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#ifndef _TC_TM_SAMPLE_H_
#define _TC_TM_SAMPLE_H_


/******************************************************************************/
/*                Definition of exported macros                 			  */
/******************************************************************************/

/**
 *  @brief  TCTM Encoder Decoder API Execution status macros
 */
#define TCTM_OK                   (0)      ///< Success return value
#define TCTM_NULL_PTR             (-1)     ///< Null pointer return value




/******************************************************************************/
/*         Definition of exported types (typedef, enum, struct, union)        */
/******************************************************************************/

/*!
 *  @brief This enum holds the enumeration values of TC and TM ID
 */
typedef enum _tc_tm_id_ft
{

	/*
	 *
	// File Transfer
    TC_FTCI = 100,
    TM_FTSR = 101,
    TC_FTDS = 102,
    TC_FTFCI = 103,

    TM_FTCI = 104,
    TC_FTSR = 105,
    TM_FTDS = 106,
    TM_FTFCI = 107,
	 *
	 */



	FT_TC_FTCI = 100,   // GS sender
	FT_TM_FTSR,  // SAT response to GS
	FT_TC_FTDS,   // GS sender
	FT_TC_FTFCI,  // GS sender

	FT_TM_FTCI,  // SAT sender
	FT_TC_FTSR,   // GS response to SAT
	FT_TM_FTDS,  // SAT sender
	FT_TM_FTFCI, // SAT sender

	FT_TC_FTDB   = 563,
	FT_TM_FTDR    = 564,

	FT_TC_TM_MAX_ID = 0

}tc_tm_id_e_ft;



/**
 *  *  @brief  Data Type definitions
 *   */
typedef unsigned int uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;

typedef int int32;
typedef short int16;
typedef char int8;




#endif
