/**
 * @file exo_ral_cmsis_mem.c
 *
 * @brief This file contains wrapper functions of CMSIS RTOS v2 for queue management.
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,s
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "exo_os_common.h"
#include "FreeRTOS.h"

#include "exo_osal_mem_management.h"
#include "exo_ral_mem_management.h"

/*!
 * @brief  wrapper function for dynamic memory allocation in eram
 */
void* ral_cmsis_mem_alloc_eram(uint32_t bytes)
{
	void* ptr =NULL;
	ptr = (void *)pvPortMalloc_eram(bytes);
	return ptr;
}

/*!
 * @brief  wrapper function for dynamic memory allocation
 */
void* ral_cmsis_mem_alloc(uint32_t bytes)
{
	void* ptr =NULL;
	ptr = (void *)pvPortMalloc(bytes);
	return ptr;
}

/*!
 * @brief wrapper function to free the memory
 */
void ral_cmsis_mem_free(void* ptr)
{
    vPortFree(ptr);
}
