/**
 * @file comms_uhf_tmr.h
 *
 * @brief This file has the control blocks, enumeration and function prototypes of SDR control layer of S-Band module
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef COMMS_UHF_TMR_H
#define COMMS_UHF_TMR_H

#include "comms_uhf_main.h"


#ifdef __cplusplus
extern "C" {
#endif

int8_t comms_uhf_timer_init(void);

/**
* @brief This api Initialize the SBAND sync.
*/
void uhf_initial_sync_cb(void);

/**
 * @brief call back function at short duration expiry
 * pass fsm state via IPC
 *
 */
void uhf_rx_on_timer_cb(void);

/**
* @brief This api Initialize the SBAND rx sync.
*/
void uhf_tm_periodic_stor_cb(void);

void uhf_cmd_timeout_cb(void);

/**
 * @brief This function is the callback for uhf window tx on timer expiry
 */
void uhf_tx_win_on_cb(void);
/**
* @brief This API monitor tx uhf ground contact
*/
void uhf_tx_tm_on_cb(void);

/**
* @brief Callback function for beacon timer expiry
*/

void uhf_beacon_timer_exp_cb(void);

void uhf_beacon_enb_timer_cb(void);

void uhf_tm_periodic_read_cb(void);

void uhf_beacon_rep_timer_cb(void);

/**
* @brief This function handles the timer expiry of transmit sync
*
* @param ptr : unused pointer
*/
void uhf_beacon_timer_cb_tmr_exp(void *ptr);

void uhf_rsp_timeout_recovery_cb(void);

void uhf_cmd_wait_tmr_exp_cb(void);

#ifdef __cplusplus
}
#endif
#endif /*COMMS_UHF_TMR_H */
