/**
 * @file otsu_def.h
 *
 * @brief This file contains type defines of OTSU
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef OTSU_DEF_H_
#define OTSU_DEF_H_

#include "stm32f7xx_hal.h"


#define NEXT_API_ADDRESS                   4u
#define OTSU_CTRL_STRUCT_SIZE 24


typedef enum suspend_def
{
	suspend_deactivated = 0x10,
	suspend_activated
}e_suspend_def;


typedef enum usage_mon_def
{
	api_inuse = 0x10,
	api_notuse
}e_usage_mon_def;


// otsu run-time control information
/* size is used in function ModuleName_get_suspend_status */
typedef struct otsu_rtl_control_def
{
	 e_suspend_def e_suspend_flag;
	 uint32_t 		api_usage_counter;
	 void          (*set_suspend_fp)(void);
	 void          (*clear_suspend_fp)(void);
	 uint32_t       (*get_suspend_status_fp)(void);
} s_otsu_rtl_ctrl_def;


// Structure for dummy api
typedef struct module_dummy_api_def
{
	void   (*module_dummy_api_fp)(void);
	uint8_t api_usage_monitor;

}s_module_dummy_api_def;


#endif /* OTSU_DEF_H_ */
