/*
******************************************************************************
**
** @file        : LinkerScript.ld
**
** 
******************************************************************************
*/

/* Entry Point */
ENTRY(Reset_Handler)

/* Highest address of the user mode stack */
_estack = ORIGIN(RAM) + LENGTH(RAM); /* end of "RAM" Ram type memory */

_Min_Heap_Size = 0x200 ; /* required amount of heap */
_Min_Stack_Size = 0x400 ; /* required amount of stack */


_static_ram_alloc_addrs = 0x20003000-1;
_ld_vtor_ram_address_set = 0x20003000;
/* Memories definition */
MEMORY
{

/* ------------------------------------------------------------------------------------Bootloader RAM region ( 14KB ) */  
/* Jumptable memory | per module jump table size 64*4 = 256B, so for 48 modules requires  12KB size */
  BL_JUMPTBL_RAM  (rx)      : ORIGIN = 0x20000000,                                              LENGTH = 1K
  
/* Got table memory | per module got table size 64*4 = 256B, so for 48 modules requires  12KB size */
  BL_GOT_RAM      (xrw)     : ORIGIN = 0x20000400,                                              LENGTH = 1K

/* Bootloader software memory region ( 12KB )*/
  BL_EXTERN_RAM     (xrw)   : ORIGIN = 0x20000800,                                              LENGTH = 1K  
  BL_RAM            (xrw)   : ORIGIN = 0x20000C00,                                              LENGTH = 11K  



/* ------------------------------------------------------------------------------------EXO RAM region  ( 498 KB) */  
  VTOR_RAM      (xrw)      : ORIGIN = 0x20003800,                                               LENGTH = 1K
  ETH_RX_DES	(xrw)      : ORIGIN = 0x20003C00, 												LENGTH = 0x200
  ETH_TX_DES	(xrw)      : ORIGIN = 0x20003E00, 												LENGTH = 0x200
  RAM           (xrw)      : ORIGIN = (ORIGIN(ETH_TX_DES) + LENGTH(ETH_TX_DES)),                LENGTH = 496K
/* RAM starts at 0x20009200 */
  

/* ------------------------------------------------------------------------------------Bootloader FLASH region  */    
  BL_LLD_FLASH   (rx)  	   : ORIGIN = 0x8000000,                                                LENGTH = 32K  
  BL_FLASH  	(rx)  	   : ORIGIN = ORIGIN(BL_LLD_FLASH) + LENGTH(BL_LLD_FLASH),              LENGTH = 26K
  BL_EXTERN_FLASH   (rx)   : ORIGIN = ORIGIN(BL_FLASH) + LENGTH(BL_FLASH),                      LENGTH = 6K


/* ------------------------------------------------------------------------------------EXO FLASH region        */    
  BIN_HEADER_FLASH (rx)    : ORIGIN = 0x8010000,      											LENGTH = 48
  MODULE_INIT_FLASH  (rx)  : ORIGIN = (ORIGIN(BIN_HEADER_FLASH) + LENGTH(BIN_HEADER_FLASH)),    LENGTH = 16  
  ISR_VCTOR_FLASH (rx)     : ORIGIN = (ORIGIN(MODULE_INIT_FLASH) + LENGTH(MODULE_INIT_FLASH)),  LENGTH = 512
  FLASH    (rx)            : ORIGIN = (ORIGIN(ISR_VCTOR_FLASH) + LENGTH(ISR_VCTOR_FLASH)),  	LENGTH = 850K   



/* ------------------------------------------------------------------------------------External Memory region  */       
  E_SDRAM    		 (xrw)   : ORIGIN = 0xC0200000,   	LENGTH = 30M
  
  E_NVM_FLASH_FMC   (rx)    : ORIGIN = 0x60000000,    	LENGTH = 768K
  E_NVM_FLASH_QSPI  (rx)    : ORIGIN = 0x90000000,    	LENGTH = 768K
  E_FMC_NORFLASH     (rx)    : ORIGIN = 0x600C0000,    	LENGTH = 13568K
  E_QSPI_NORFLASH    (rx)    : ORIGIN = 0x900C0000,   	LENGTH = 62720K
  E_OTSU_FLASH_FMC   (rx)    : ORIGIN = 0x60E00000,    	LENGTH = 2M
  E_OTSU_FLASH_QSPI  (rx)    : ORIGIN = 0x93E00000,    	LENGTH = 2M  
  E_SDRAM_TCTM		 (xrw)   : ORIGIN = 0xC0000000,   	LENGTH = 2M
}

/* Sections */
SECTIONS
{



  . = ALIGN(4);
  .hdrinfo :
  {
    . = ALIGN(4); 
    *(.hdrinfo)         
    *(.hdrinfo*)        
    . = ALIGN(4);
  } >BIN_HEADER_FLASH
  


  .module_init_section :
  {
    . = ALIGN(4);
    KEEP(*(.module_init_section)) /* Startup code */
    . = ALIGN(4);
  } >MODULE_INIT_FLASH




  /* The startup code into "FLASH" Rom type memory */
  .isr_vector :
  {
    . = ALIGN(4);
    KEEP(*(.isr_vector)) /* Startup code */
    . = ALIGN(4);
  } >ISR_VCTOR_FLASH

  /* The program code and other data into "FLASH" Rom type memory */
  .text :
  {
    . = ALIGN(4);
    *(.text)           /* .text sections (code) */
    *(.text*)          /* .text* sections (code) */
    *(.glue_7)         /* glue arm to thumb code */
    *(.glue_7t)        /* glue thumb to arm code */
    *(.eh_frame)

    KEEP (*(.init))
    KEEP (*(.fini))

    . = ALIGN(4);
    _etext = .;        /* define a global symbols at end of code */
  } >FLASH

  /* Constant data into "FLASH" Rom type memory */
  .rodata :
  {
    . = ALIGN(4);
    *(.rodata)         /* .rodata sections (constants, strings, etc.) */
    *(.rodata*)        /* .rodata* sections (constants, strings, etc.) */
    . = ALIGN(4);
  } >FLASH

  .ARM.extab   : {
    . = ALIGN(4);
    *(.ARM.extab* .gnu.linkonce.armextab.*)
    . = ALIGN(4);
  } >FLASH

  .ARM : {
    . = ALIGN(4);
    __exidx_start = .;
    *(.ARM.exidx*)
    __exidx_end = .;
    . = ALIGN(4);
  } >FLASH

  .preinit_array     :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__preinit_array_start = .);
    KEEP (*(.preinit_array*))
    PROVIDE_HIDDEN (__preinit_array_end = .);
    . = ALIGN(4);
  } >FLASH

  .init_array :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__init_array_start = .);
    KEEP (*(SORT(.init_array.*)))
    KEEP (*(.init_array*))
    PROVIDE_HIDDEN (__init_array_end = .);
    . = ALIGN(4);
  } >FLASH

  .fini_array :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__fini_array_start = .);
    KEEP (*(SORT(.fini_array.*)))
    KEEP (*(.fini_array*))
    PROVIDE_HIDDEN (__fini_array_end = .);
    . = ALIGN(4);
  } >FLASH

  /* Used by the startup to initialize data */
  _sidata = LOADADDR(.data);

  /* Initialized data sections into "RAM" Ram type memory */
  .data :
  {
    . = ALIGN(4);
    _sdata = .;        /* create a global symbol at data start */
    *(.fixed_data)  
    *(.data)           /* .data sections */
    *(.data*)          /* .data* sections */
    *(.RamFunc)        /* .RamFunc sections */
    *(.RamFunc*)       /* .RamFunc* sections */

    . = ALIGN(4);
    _edata = .;        /* define a global symbol at data end */
  } >RAM AT> FLASH

  /* Uninitialized data section into "RAM" Ram type memory */
  . = ALIGN(4);
  .bss :
  {
    /* This is used by the startup in order to initialize the .bss section */
    _sbss = .;         /* define a global symbol at bss start */
    __bss_start__ = _sbss;
    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(4);
    _ebss = .;         /* define a global symbol at bss end */
    __bss_end__ = _ebss;
  } >RAM

  /* User_heap_stack section, used to check that there is enough "RAM" Ram  type memory left */
  ._user_heap_stack :
  {
    . = ALIGN(8);
    PROVIDE ( end = . );
    PROVIDE ( _end = . );
    . = . + _Min_Heap_Size;
    . = . + _Min_Stack_Size;
    . = ALIGN(8);
  } >RAM


  /* Used by the application software to know bootloader api address fptr */
/*  _sibtlr_api_addr = 0x20000000; */
_sibtlr_api_addr = LOADADDR(.btlr_api_fptr_addr);


  .btlr_api_fptr_addr :
  {
    . = ALIGN(4);
    *(.btlr_api_fptr_addr)           
    *(.btlr_api_fptr_addr*)          
    . = ALIGN(4);
  } >BL_EXTERN_RAM AT> FLASH



  .ext_qspiflash_firmware (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_qspiflash_firmware)       /* .ext_qspiflash_firmware sections */
  } >E_OTSU_FLASH_QSPI

  .ext_fmcflash_firmware (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_fmcflash_firmware)       /* .ext_fmcflash_firmware sections */
  } >E_OTSU_FLASH_FMC

  .ext_qspiflash_nvm (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_qspiflash_nvm)       /* .ext_qspiflash_nvm sections */
  } >E_NVM_FLASH_QSPI

  .ext_fmcflash_nvm (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_fmcflash_nvm)       /* .ext_fmcflash_nvm sections */
  } >E_NVM_FLASH_FMC


  .ext_ram_data (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_ram_data)           		/* .ext_ram_data sections */
  } >E_SDRAM

  .ext_ram_data_tctm (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_ram_data_tctm)           		/* .ext_ram_data_tc sections */
  } >E_SDRAM_TCTM

  .ext_qspiflash_data (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_qspiflash_data)           /* .ext_qspiflash_data sections */
  } >E_QSPI_NORFLASH

  .ext_fmcflash_data (NOLOAD)  :
  {
  	. = ALIGN(4);
    *(.ext_fmcflash_data)           /* .ext_fmcflash_data sections */
  } >E_FMC_NORFLASH

  /* Remove information from the compiler libraries */
  /DISCARD/ :
  {
    libc.a ( * )
    libm.a ( * )
    libgcc.a ( * )
  }



  .ARM.attributes 0 : { *(.ARM.attributes) }
  .RxDecripSection (NOLOAD) : { *(.RxDescripSection) } >ETH_RX_DES
  .TxDescripSection (NOLOAD) : { *(.TxDecripSection) } >ETH_TX_DES
  
}
