/**
 * @file mfcc_csp_intf.h
 *
 * @brief This file contains function prototypes and types that is used
 *        for sending and receiving data through can interface
 */

#ifndef MFCC_CSP_INTF_H_
#define MFCC_CSP_INTF_H_

#include <stdint.h>
#include <errno.h>


#define SRS4_SPACE_ADDRESS 11
#define SRS4_CTRL_ADDRESS 24

#define SRS4_CAN_MTU 1400
#define SRS4_ETH_MTU 1020

#define SRS4_DEFAULT_TIMEOUT 1000

void CSP_Setup(void);

/**
 * @brief This api transmits the tm data of length size
 *  through the can interface
 *
 */
int32_t csp_send_tm(uint8_t* tm_data, uint16_t len);

/**
 * @brief This receives the tc/tm data through the can interface
 * and sends the TC/TM payload to TC-TM manager through IPC
 *
 */
int32_t csp_recv_tc(void);
int csp_slgnd_init_udp(const char *radio_host, const char *ifname, csp_iface_t **ifc);

#endif /* MFCC_CSP_INTF_H_ */
