#ifndef _MFCC_TC_TM_SAMPLE_H_
#define _MFCC_TC_TM_SAMPLE_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tc_common.h>
#include <pthread.h>
#include <tc_tm_lib_common.h>
#include <comms_al.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "mfcc_sample_app.h"
#include "mfcc_tc_tm_sample.h"
#include "tm_common.h"

/**
 *  *  @brief  Data Type definitions
 *   */
typedef unsigned int uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;

typedef int int32;
typedef float f32;
typedef short int16;
typedef char int8;

/**
 *  @brief  TCTM Encoder Decoder API Execution status macros
 */
#define TCTM_OK                   (0)      ///< Success return value
#define TCTM_NULL_PTR             (-1)     ///< Null pointer return value

//#define ENBL_DBG_PRNT                              ///< Macro for enabling debug prints

/// TC Socket interface parameter configurations
#define MFCC_TC_SOCKET_IP_ADDR      "192.168.10.2" ///< IP address of socket used for TC transmission
#define MFCC_TC_SOCKET_PORT_ADDR    5001           ///< Port address of socket used for TC transmission

/// TM Socket interface parameter configurations
#define MFCC_TM_SOCKET_IP_ADDR      "192.168.10.2" ///< IP address of socket used for TM reception
#define MFCC_TM_SOCKET_PORT_ADDR    5002           ///< Port address of socket used for TM transmission

/// TC_TM Frame Parameter configurations
#define MAX_TC_BUFF_SIZE            1024           ///< Maximum TC buffer size allocation
#define MAX_TM_BUFF_SIZE            1024           ///< Maximum TM buffer size allocation
#define TC_FRAME_SOF                0xA5           ///< Start of frame definition for TC
#define RECV_SAT_ID                 0x01           ///< Satellite ID value
#define CRYPTO_ENCRYP_ENBL          0x20           ///< Encryption configuration value
#define SOURCE_GROUND_STN_ID        0x01           ///< Ground station ID value
#define TC_QOS_CFG                  0x01           ///< TC_TM Quality of Service configuration value

#define TC_SRC_GS                   0x01           ///< TC Source ID configuration value for Ground Station
#define TC_DST_OBC                  0x81           ///< TC Destination ID configuration value for OBC
#define TC_DST_PS                   0x86           ///< TC Destination ID configuration value for PS
#define TC_DST_SBND_COMMS           0x84           ///< TC Destination ID configuration value for COMMS - Sband
#define TC_DST_ADCS                 0x82           ///< TC Destination ID configuration value for ADCS

#define TM_SRC_OBC                  0x81           ///< TM Source ID configuration value for OBC
#define TM_SRC_PS                   0x86           ///< TM Source ID configuration value for PS
#define TM_SRC_SBND_COMMS           0x84           ///< TM Source ID configuration value for COMMS - Sband
#define TM_DST_GS                   0x01           ///< TM Destination ID configuration value for GS

/*!
 *  @brief This enum holds the enumeration values of TC and TM ID
 */
typedef enum _tc_tm_id
{
    // TODO - Enumeration value to be updated after discussion with team
    TC_TRGT_PT = 55,
    TM_TRGT_PT = 55,

    // File Transfer
    TC_FTCI = 100,
    TM_FTSR = 101,
    TC_FTDS = 102,
    TC_FTFCI = 103,

    TM_FTCI = 104,
    TC_FTSR = 105,
    TM_FTDS = 106,
    TM_FTFCI = 107,

    // TTY Debug
    TC_TTY_START = 116,
    TM_TTY_START = 116,

    TC_TTY_DEBUG = 117,
    TM_TTY_DEBUG = 117,

    TC_TTY_END = 118,
    TM_TTY_END = 118,
    // TBD: This is the ID provided by ADCS vendor
    TC_LOC_INFO = 150,
    TM_LOC_INFO = 150,

    // EPS
    TC_SET_CONV_BOARD_PARM = 200,
    TM_SET_CONV_BOARD_PARM = 200,

    TC_GET_CONV_BOARD_PARM = 201,
    TM_GET_CONV_BOARD_PARM = 201,

    TC_SET_BATT_BOARD_PARM =202,
    TM_SET_BATT_BOARD_PARM =202,

    TC_GET_BATT_BOARD_PARM =203,
    TM_GET_BATT_BOARD_PARM =203,

    TC_SET_OUT_CHNL_ENB =204,
    TM_SET_OUT_CHNL_ENB =204,

    TC_SET_OUT_CHNL_CUR_LMT =205,
    TM_SET_OUT_CHNL_CUR_LMT =205,

    TC_GET_OUT_CHNL_PARM =206,
    TM_GET_OUT_CHNL_PARM =206,

    TC_SET_EPS_MODE_CFG = 207,
    TM_SET_EPS_MODE_CFG = 207,

    TC_GET_EPS_MODE_CFG =208,
    TM_GET_EPS_MODE_CFG =208,

    // COMMS 
    TC_SET_COMMS_TX_CFG = 300,
    TM_SET_COMMS_TX_CFG = 300,

    TC_SET_COMMS_RX_CFG = 319,
    TM_SET_COMMS_RX_CFG = 319,

    TC_SET_COMMS_SYS_CFG = 340,
    TM_SET_COMMS_SYS_CFG = 340,

    TC_GET_COMMS_TX_CFG = 350,
    TM_GET_COMMS_TX_CFG = 350,

    TC_GET_COMMS_RX_CFG = 369,
    TM_GET_COMMS_RX_CFG = 369,

    TC_GET_COMMS_SYS_CFG = 390,
    TM_GET_COMMS_SYS_CFG = 390,

    // Interface
    TC_SET_UART_INTF_PARM = 509,
    TM_SET_UART_INTF_PARM = 509,

    TC_GET_UART_INTF_PARM = 510,
    TM_GET_UART_INTF_PARM = 510,

    TC_SET_USB_INTF_PARM = 511,
    TM_SET_USB_INTF_PARM = 511,

    TC_GET_USB_INTF_PARM = 512,
    TM_GET_USB_INTF_PARM = 512,

    TC_SET_I2C_INTF_PARM = 513,
    TM_SET_I2C_INTF_PARM = 513,

    TC_GET_I2C_INTF_PARM = 514,
    TM_GET_I2C_INTF_PARM = 514,

    TC_SET_CAN_INTF_PARM = 515,
    TM_SET_CAN_INTF_PARM = 515,

    TC_GET_CAN_INTF_PARM = 516,
    TM_GET_CAN_INTF_PARM = 516,

    TC_SET_ETH_INTF_PARM = 517,
    TM_SET_ETH_INTF_PARM = 517,

    TC_GET_ETH_INTF_PARM = 518,
    TM_GET_ETH_INTF_PARM = 518,

    TC_SET_SPI_INTF_PARM = 519,
    TM_SET_SPI_INTF_PARM = 519,

    TC_GET_SPI_INTF_PARM = 520,
    TM_GET_SPI_INTF_PARM = 520,

    // Sensors
    TC_SET_TEMP_SEN_PARM = 524,
    TM_SET_TEMP_SEN_PARM = 524,

    TC_GET_TEMP_INFO = 525,
    TM_GET_TEMP_INFO = 525,

    TC_SET_IMU_PARM = 526,
    TM_SET_IMU_PARM = 526,

    TC_GET_IMU_INFO = 527,
    TM_GET_IMU_INFO = 527,

    TC_SET_THRUST_CTRL_PARM = 528,
    TM_SET_THRUST_CTRL_PARM = 528,

    TC_GET_THRUST_CTRL_INFO = 529,
    TM_GET_THRUST_CTRL_INFO = 529,

    TC_SET_GPIO_EXP_PARM = 530,
    TM_SET_GPIO_EXP_PARM = 530,

    TC_GET_GPIO_EXP_INFO = 531,
    TM_GET_GPIO_EXP_INFO = 531,

    TC_SET_DIG_THERM_PARM = 532,
    TM_SET_DIG_THERM_PARM = 532,

    TC_GET_DIG_THERM_INFO = 533,
    TM_GET_DIG_THERM_INFO = 533,

    TC_SET_PWR_MON_PARM = 534,
    TM_SET_PWR_MON_PARM = 534,

    TC_GET_PWR_MON_INFO = 535,
    TM_GET_PWR_MON_INFO = 535,

    TC_SET_VOLT_CTRL_MON_PARM = 536,
    TM_SET_VOLT_CTRL_MON_PARM = 536,

    TC_GET_VOLT_CTRL_MON_INFO = 537,
    TM_GET_VOLT_CTRL_MON_INFO = 537,

    TC_SET_PWR_SEQ_MON_PARM =538,
    TM_SET_PWR_SEQ_MON_PARM =538,

    TC_GET_PWR_SEQ_MON_INFO = 539,
    TM_GET_PWR_SEQ_MON_INFO = 539,

    TC_SET_DATA_LOG_PARM = 540,
    TM_SET_DATA_LOG_PARM = 540,

    TC_GET_DATA_LOG_INFO =541,
    TM_GET_DATA_LOG_INFO =541,

    // Payload Server Power Controls
    TC_PS_PWR_ON = 542,
    TM_PS_PWR_ON = 542,

    TC_PS_PWR_OFF = 543,
    TM_PS_PWR_OFF = 543,

    TC_PLD_HW_PWR_ON = 544,
    TM_PLD_HW_PWR_ON = 544,

    TC_PLD_HW_PWR_OFF = 545,
    TM_PLD_HW_PWR_OFF = 545,

    TC_PLD_VM_PWR_ON = 546,
    TM_PLD_VM_PWR_ON = 546,

    TC_PLD_VM_PWR_OFF = 547,
    TM_PLD_VM_PWR_OFF = 547,

    TC_SCHEDULE_CFG_LUT = 548,
    TM_SCHEDULE_CFG_LUT = 548,

    TC_SCHEDULE_DEPLOY = 549,
    TM_SCHEDULE_DEPLOY = 549,

    TC_TM_MAX_ID

}tc_tm_id_e;

typedef enum
{
    INDONESIAN_BRDR_LAT = 1,
    OGS_SINGAPORE_LAT,
    MAX_ADCS_PT_LOC

}adcs_pt_info_e;

typedef enum
{
    //GENERAL EVENT
    ADCS_POINT               =  1,
    POWER_ON_PLD,
    BOOT_UP_PLD_VM,
    SHUTDOWN_PLD_VM,

    //NETRA EVENTS
    NETRA_IOT_MODE_A         = 21,
    NETRA_IOT_MODE_B,
    NETRA_UPGRADE_FW,
    NETRA_UPGRADE_APP,

    //TC EVENTS
    TC_CAMPAIGN0             =  41,
    TC_CAMPAIGN1,
    TC_CAMPAIGN2,
    TC_CAMPAIGN3,
    TC_UPGRADE_FW,
    TC_UPGRADE_APP,

    //ZES EVENTS
    ZES_VOTER_NORMAL_MODE     = 61,
    ZES_VOTER_DEBUG_MODE,
    ZES_LDAP_NORMAL_MODE,
    ZES_UPGRADE_FW,
    ZES_UPGRADE_APP,

    //SAYARILABS
    SYLB_HOUSEKEEPING_SEQUENCE = 81,
    SYLB_UPGRADE_FW,
    SYLB_UPGRADE_APP,
    RESERVED

}scheduler_evnt_id;

typedef struct __attribute__((packed))
{
    uint8 min;
    uint8 sec;
}sch_time_s;

typedef struct __attribute__((packed))
{
    uint8 event_id;
    sch_time_s event_exec_time;
    uint32 event_arg;
}schedule_event_s;

typedef struct __attribute__((packed))
{
    uint8 date;
    uint8 month;
    uint16 year;
    uint8 orbit_lut_map[16];
}orbit_lut_cfg_s;

/**
  * @brief comms transmit configuration Structure definition
  *
  */
typedef struct  __attribute__((packed)){

    uint32  tx_cfg_freq;           /*!< Transmit frequency                                            */
    uint8   tx_cfg_type;           /*!< Transmit modulation type (0 = BPSK, 1 = QPSK, 2 = 8PSK)       */
    uint32  tx_cfg_rate;           /*!< Transmit symbol rate                                          */
    uint8   tx_cfg_rolloff;        /*!< RRC roll-off (0 = 0.20,  1 = 0.25,  2 = 0.35)                 */
    f32     tx_cfg_pout;           /*!< Transmit power                                                */
    uint8   tx_cfg_alc_mode;       /*!< Automatic Level Control mode (0 = open-loop, 1 = closed-loop) */
    uint8   tx_cfg_rs;             /*!< Enable Reed-Solomon coding                                    */
    uint8   tx_cfg_cc;             /*!< Enable convolutional coding                                   */
    uint8   tx_cfg_rand;           /*!< Enable randomization                                          */
    uint8   tx_cfg_crc;            /*!< Enable CRC checksum                                           */
    uint16  tx_cfg_idletime;       /*!< Active transmitter time after each frame                      */
    uint16  tx_cfg_size;           /*!< Maximum payload size                                          */
    uint16  tx_cfg_id;             /*!< Satellite ID/address                                          */
    uint8   tx_cfg_crypto_key[32]; /*!< Transmit Encryption key                                       */
    uint8   tx_cfg_crypto_encrypt; /*!< Transmit Encrypt frames                                       */
    uint8   tx_cfg_crypto_auth;    /*!< Add authentication tag                                        */
    uint8   tx_cfg_pa_templimit;   /*!< PA temperature limit                                          */
    uint32  tx_cfg_frames;         /*!< Transmitted frames                                            */
    f32     tx_cfg_pwr_fwd;        /*!< Measured forward power                                        */
    f32     tx_cfg_pwr_rfl;        /*!< Transmit the Measured reflected power                         */
}s_comms_tx_cfg;

/**
  * @brief comms receive configuration Structure definition
  *
  */
typedef struct  __attribute__((packed))
{
    uint32  rx_cfg_freq;          /*!< Receive frequency                             */
    uint8   rx_cfg_type;          /*!< Receive modulation type (0 = BPSK,  1 = QPSK) */
    uint32  rx_cfg_rate;          /*!< Receive symbol rate                           */
    uint8   rx_cfg_rolloff;       /*!< RRC roll-off (0 = 0.20, 1 = 0.25, 2 = 0.35)   */
    uint8   rx_cfg_afc_enable;    /*!< Enable AFC                                    */
    uint32  rx_cfg_afc_range;     /*!< AFC adjustment range                          */
    uint8   rx_cfg_rs;            /*!< Enable Reed-Solomon coding                    */
    uint8   rx_cfg_cc;            /*!< Enable convolutional coding                   */
    uint8   rx_cfg_rand;          /*!< Enable randomization                          */
    uint8   rx_cfg_crc;           /*!< Enable CRC checksum                           */
    uint16  rx_cfg_size;          /*!< Maximum payload size                          */
    uint16  rx_cfg_id;            /*!< Satellite ID/address                          */
    uint8   rx_cfg_crypto_key[32];/*!< Decryption key                                */
    uint8   rx_cfg_crypto_decrypt;/*!< Decrypt frames                                */
    uint8   rx_cfg_crypto_auth;   /*!< Verify authentication tag                     */
    uint32  rx_cfg_frames;        /*!< Received frames                               */
    uint32  rx_cfg_detected;      /*!< Detected frames                               */
    f32     rx_cfg_rssi;          /*!< Measured RSSI                                 */
    f32     rx_cfg_freqerr;       /*!< Estimated frequency error                     */
    f32     rx_cfg_symerr;        /*!< Estimated symbol rate error                   */
    f32     rx_cfg_ebn0;          /*!< Estimated Eb/N0                               */
    uint8   rx_cfg_carrier_lock;  /*!< comms receive config carrier lock             */
    uint8   rx_frame_lock;        /*!< comms receive frame lock                      */
}s_comms_rx_cfg;

/**
  * @brief comms system details in Structure definition
  *
  */
typedef struct  __attribute__((packed))
{
    int8      sys_name[20];         /*!< system String name for system         */
    uint32    sys_serial;           /*!< Serial number/ID                      */
    uint32    sys_sw_version;       /*!< Running software version              */
    uint32    sys_bootcount;        /*!< Number of times the system was booted */
    uint16    sys_reboot;           /*!< Reboot timer                          */
    uint32    sys_gwdt_init;        /*!< GWDT timer reset value                */
    uint32    sys_gwdt_counter;     /*!< GWDT current counter value            */
    uint32    sys_gwdt_reset;       /*!< GWDT reset trigger                    */
    uint8     sys_csp_address;      /*!< csp address of the device             */
    int8      sys_csp_routes[100];  /*!< csp routes in CIDR notation           */
    uint32    sys_can_rate;         /*!< CAN-bus bitrate                       */
    uint8     sys_can_promisc;      /*!< Accept all CSP CAN-frames             */
    uint32    sys_uart_rate;        /*!< UART baudrate                         */
    uint8     sys_uart_multidrop;   /*!< UART multidrop mode                   */
    uint8     sys_net_enable;       /*!< IP stack enable                       */
    int8      sys_net_en_ip[50];    /*!< Ethernet IP address                   */
    uint8     sys_net_en_prefix;    /*!< Ethernet network prefix bits          */
    int8      sys_net_en_gw[50];    /*!< Ethernet gateway IP address           */
    int8      sys_net_rf_ip[50];    /*!< Spacelink IP address                  */
    uint8     sys_net_rf_prefix;
    int8      sys_net_rf_gw[50];

}s_comms_sys_cfg;

typedef struct
{
    uint32_t instance_id;
    int16_t high_temp_thrsld;
    int16_t low_temp_thrsld;
    uint8_t po_pin_logic;
    uint8_t adc_resolution;
    uint8_t operating_mode;
    int16_t temperature;

}__attribute__ ((__packed__))digital_thermostat_info;

typedef struct
{
    uint32_t instance_id;
    int16_t high_temp_thrsld;
    int16_t low_temp_thrsld;
    uint8_t po_pin_logic;
    uint8_t adc_resolution;
    uint8_t operating_mode;

}__attribute__ ((__packed__))digital_thermostat_param;


typedef struct
{
    uint32_t instance_id;
    uint16_t dvc_id;
    uint16_t manufacture_id;
    uint16_t temperature;
    uint16_t high_temp_thrsld;
    uint16_t low_temp_thrsld;
    uint16_t critical_temp_thrsld;
    uint8_t adc_resolution;
    uint8_t operating_mode;
    uint8_t polarity;
    uint8_t hysteresis;
    uint8_t window_lock;
    uint8_t event_mode;

}__attribute__ ((__packed__))temp_sens_info;

typedef struct
{
    uint32_t instance_id;
    uint16_t high_temp_thrsld;
    uint16_t low_temp_thrsld;
    uint16_t critical_temp_thrsld;
    uint8_t adc_resolution;
    uint8_t operating_mode;
    uint8_t polarity;
    uint8_t hysteresis;
    uint8_t window_lock;
    uint8_t event_mode;

}__attribute__ ((__packed__))temp_sens_param;

typedef struct
{
    uint32_t instance_id;
    uint8_t intr_pin_polarity;

}__attribute__ ((__packed__))gpio_exp_info;

typedef struct
{
    uint32_t instance_id;
    uint8_t intr_pin_polarity;

}__attribute__ ((__packed__))gpio_exp_param;

typedef struct
{
    uint32_t instance_id;
    float voltage;
    float current;
    uint8_t ovr_crnt_sts;
    uint8_t hotswap_sts;

}__attribute__ ((__packed__))hotswap_cntlr_info;

typedef struct
{
    uint32_t instance_id;
    uint8_t hotswap_sts;

}__attribute__ ((__packed__))hotswap_cntlr_param;

typedef struct
{
    uint32_t instance_id;
    uint16_t vbus_thrsld;
    int16_t vshunt_thrsld;
    uint16_t calibration;
    float vbus_voltage;
    float current;
    uint16_t power;

}__attribute__ ((__packed__))power_sense_mntr_info;

typedef struct
{
    uint32_t instance_id;
    uint16_t vbus_thrsld;
    int16_t vshunt_thrsld;
    uint16_t calibration;

}__attribute__ ((__packed__))power_sense_mntr_param;

typedef struct
{
    uint32_t instance_id;
    uint8_t flash_lock_sts;
    uint8_t v_ref_type;

}__attribute__ ((__packed__))voltage_seq_mntr_info;

typedef struct
{
    uint32_t instance_id;
    uint8_t flash_lock_sts;
    uint8_t v_ref_type;

}__attribute__ ((__packed__))voltage_seq_mntr_param;

typedef struct
{
    uint32_t instance_id;
    uint8_t mjr_vrsn;
    uint8_t mnr_vrsn;
    uint16_t ptch_vrsn;
    uint8_t brd_err_cde;
    uint8_t brd_err_count;
    uint8_t rst_sts;
    uint32_t strt_tm;
    uint32_t master_tm;
    uint32_t bkpln_master_tm;			/*!< master time of backplane			*/
    uint8_t rst_backplane;		    	/*!< backplane reset			    	*/
    uint16_t pwr;		    		/*!< power		    			*/
    uint16_t unreg_vol;			    	/*!< unregulated voltage			*/
    uint32_t unit_master_tm;			/*!< master time of unit			*/
    uint8_t mode;		    		/*!< mode		    			*/
    uint16_t dev_on_sts;		    	/*!< device on status				*/
    uint8_t rst_sts_unit;		    	/*!< reset status of unit		    	*/
    int8_t temp;		    		/*!< temperature	    			*/
    uint16_t prplt;		    		/*!< propellant		    			*/
    uint16_t force;		    		/*!< force		    			*/
    uint16_t azmth; 				/*!< azimuthal angle		    		*/
    uint8_t polar;		    		/*!< polar angle		    		*/
    uint16_t unregu_vol;		    	/*!< unregulated voltage		    	*/
    uint32_t calib_data[7]; 			/*!< calibration value		    		*/
    uint8_t heat_err;		    		/*!< heater error		    		*/
    uint8_t heat_duty;		    		/*!< heater duty cycle			    	*/
    int8_t heat_temp;		    		/*!< heater temperature			    	*/
    int8_t heat_targ_temp;		    	/*!< heater target temperature			*/
    uint16_t emit_vol;			    	/*!< emitter voltage			    	*/
    uint16_t emit_cur;			    	/*!< emitter current			    	*/
    uint16_t emit_ip_cur;	    		/*!< emitter current			    	*/
    uint16_t ext1_vol;		    		/*!< extractor1 voltage			    	*/
    uint16_t ext1_cur;		    		/*!< extractor1 current			    	*/
    uint16_t ext1_ip_cur;	    		/*!< extractor1 input current			*/
    uint16_t ext2_vol;		    		/*!< extractor2 voltage			    	*/
    uint16_t ext2_cur;		    		/*!< extractor2 current		    		*/
    uint16_t ext2_ip_cur;	    		/*!< extractor2 input current			*/
    uint16_t ext3_vol;		    		/*!< extractor3 voltage			    	*/
    uint16_t ext3_cur;		    		/*!< extractor3 voltage			    	*/
    uint16_t ext3_ip_cur;	    		/*!< extractor3 voltage			    	*/
    uint16_t ext4_vol;		    		/*!< extractor4 voltage			    	*/
    uint16_t ext4_cur;		    		/*!< extractor4 voltage			    	*/
    uint16_t ext4_ip_cur;	    		/*!< extractor4 voltage			    	*/
    uint16_t ext5_vol;		    		/*!< extractor5 voltage			    	*/
    uint16_t ext5_cur;		    		/*!< extractor5 voltage			    	*/
    uint16_t ext5_ip_cur;	    		/*!< extractor5 voltage				*/
    uint16_t ext6_vol;		    		/*!< extractor6 voltage				*/
    uint16_t ext6_cur;		    		/*!< extractor6 voltage	    			*/
    uint16_t ext6_ip_cur;   			/*!< extractor6 voltage	    			*/
    uint16_t ext7_vol;	    			/*!< extractor7 voltage	    			*/
    uint16_t ext7_cur;	    			/*!< extractor7 voltage	    			*/
    uint16_t ext7_ip_cur;   			/*!< extractor7 voltage				*/
    uint16_t neut1_vol;	    			/*!< neutralizer1 voltage	    		*/
    uint16_t neut1_cur;	    			/*!< neutralizer1 current		    	*/
    uint16_t neut1_ip_cur;  			/*!< neutralizer1 input current			*/
    uint16_t neut1_ret_cur; 			/*!< neutralizer1 return current		*/
    uint16_t neut2_vol;	    			/*!< neutralizer2 voltage	    		*/
    uint16_t neut2_cur;	    			/*!< neutralizer2 current		    	*/
    uint16_t neut2_ip_cur;	    		/*!< neutralizer2 input current			*/
    uint16_t neut2_ret_cur;	    		/*!< neutralizer2 return current		*/
    uint16_t tot_force;				/*!< total force	    		*/
    uint16_t tot_azmth;				/*!< total azimuthal angle		*/
    uint8_t tot_polar;				/*!< total polar angle		    	*/
    uint16_t tot_pwr_cnsp;		/*!< total power consumption		*/
    uint16_t force_unit1;			/*!< force of unit1     	    	*/
    uint8_t azmth_unit1;			/*!< azimuthal angle of unit1	        */
    uint16_t polar_unit1;			/*!< polar angle of unit1	    	*/
    uint16_t pwr_unit1;				/*!< power of unit1	    		*/
    uint8_t mode_unit1;				/*!< mode of unit1	    		*/
    uint16_t force_unit2;			/*!< force of unit2	    		*/
    uint8_t polar_unit2;			/*!< polar angle of unit2   		*/
    uint16_t azmth_unit2;			/*!< azimuthal angle of unit2		*/
    uint16_t pwr_unit2;				/*!< power of unit2		    	*/
    uint8_t mode_unit2;				/*!< mode of unit2  			*/

}__attribute__ ((__packed__))thruster_info;

typedef struct
{
    uint32_t instance_id;
    uint8_t unit_num;
    uint8_t mode;
    float force;

}__attribute__ ((__packed__))thruster_param;

typedef struct
{
    uint32_t instance_id;
    uint8_t chnl_num;
    uint8_t pwr_dwn_optn;
    uint16_t data;

}__attribute__ ((__packed__))data_acq_dvc_info;

typedef struct
{
    uint32_t instance_id;
    uint8_t chnl_num;
    uint8_t pwr_dwn_optn;

}__attribute__ ((__packed__))data_acq_dvc_param;

typedef struct
{
    uint32_t instance_id;
    /*! X-axis sensor data */
    int16_t accel_x;
    /*! Y-axis sensor data */
    int16_t accel_y;
    /*! Z-axis sensor data */
    int16_t accel_z;
    /*! X-axis sensor data */
    int16_t gyro_x;
    /*! Y-axis sensor data */
    int16_t gyro_y;
    /*! Z-axis sensor data */
    int16_t gyro_z;
    /*! X-axis sensor data */
    int16_t mag_x;
    /*! Y-axis sensor data */
    int16_t mag_y;
    /*! Z-axis sensor data */
    int16_t mag_z;
    /*! rhall data */
    int16_t mag_rhall;
    /*! power mode */
    uint8_t power;
    /*! output data rate */
    uint8_t odr;
    /*! range */
    uint8_t range;
    /*! bandwidth */
    uint8_t bw;

}__attribute__ ((__packed__))imu_info;

typedef struct
{
    uint32_t instance_id;
    /*! power mode */
    uint8_t power;
    /*! output data rate */
    uint8_t odr;
    /*! range */
    uint8_t range;
    /*! bandwidth */
    uint8_t bw;

}__attribute__ ((__packed__))imu_param;

/*********************************************************************************/
/***************** TeleCommand Payload Structure Definitions *********************/
/*********************************************************************************/

/*!
 *  @brief This structure holds the Target Pointing request parameter information
 */
typedef struct _tctm_trgt_pt_req
{
    float lat;
    float longit;
    float alt;
    uint8 pld_hw_id;
    uint8 act_tmr_min;
    uint8 act_tmr_sec;
    uint16 act_tmr_ms;
}tctm_trgt_pt_req_s;

/*********************************************************************************/
/******************* TeleMetry Payload Structure Definitions *********************/
/*********************************************************************************/

/*!
 *  @brief This structure holds the location information
 */
typedef struct _tctm_gnss_loc_info
{
    uint8 status;
    float lat;
    float longit;
    float alt;
}tctm_gnss_loc_info_s;



/**
 *  *   @brief UART Init Structure definition
 *   **/
typedef struct
{
    uint32_t baudrate;
    uint32_t wordlength;
    uint32_t stopbits;
    uint32_t parity;
    uint32_t mode;
    uint32_t hw_flow_ctl;
    uint32_t over_sampling;
    uint32_t onebit_sampling;
}__attribute__((__packed__))tctm_uart_init_s;


/**
 * *  @brief USB Instance Initialization Structure definition
 * */
typedef struct
{
    uint32_t dev_endpoints;
    uint32_t host_channels;
    uint32_t speed;
    uint32_t dma_enable;
    uint32_t ep0_mps;
    uint32_t phy_itface;
    uint32_t sof_enable;
    uint32_t low_power_enable;
    uint32_t lpm_enable;
    uint32_t battery_charging_enable;
    uint32_t vbus_sensing_enable;
    uint32_t use_dedicated_ep1;
    uint32_t use_external_vbus;

}__attribute__((__packed__))tctm_usb_otg_cfg_s;


/**
 *  * @defgroup I2C_Configuration_Structure_definition I2C Configuration Structure definition
 *   * @brief  I2C Configuration Structure definition
 *    *
 *     **/
typedef struct
{
    uint32_t timing;
    uint32_t own_address1;
    uint32_t addressing_mode;
    uint32_t dual_address_mode;
    uint32_t own_address2;
    uint32_t own_address2_masks;
    uint32_t generalcall_mode;
    uint32_t no_stretch_mode;
}__attribute__((__packed__))tctm_i2c_init_s;
/**
 *  *  @brief  CAN init structure definition
 *   **/
typedef struct
{
    uint32_t prescaler;
    uint32_t mode;
    uint32_t syncjumpwidth;
    uint32_t time_seg1;
    uint32_t time_seg2;
    uint8_t  time_triggered_mode;
    uint8_t  autobus_off;
    uint8_t  autowake_up;
    uint8_t  auto_retransmission;
    uint8_t  receive_fifo_locked;
    uint8_t  transmit_fifo_priority;

}__attribute__((__packed__))tctm_can_init_s;

/* 
 * @brief  ETH Init Structure definition
 */
typedef struct
{
    uint32_t  auto_negotiation;
    uint32_t  speed;
    uint32_t  duplex_mode;
    uint16_t  phy_address;
    uint8_t   *mac_addr;
    uint32_t  rx_mode;
    uint32_t  checksum_mode;
    uint32_t  media_interface;

}__attribute__((__packed__))tctm_ethernet_init_s;


/**
 *  *   @brief  SPI Configuration Structure definition
 *   **/
typedef struct
{
    uint32_t mode;
    uint32_t direction;
    uint32_t datasize;
    uint32_t clk_polarity;
    uint32_t clk_phase;
    uint32_t nss;
    uint32_t baudrate_prescaler;
    uint32_t firstbit;
    uint32_t timode;
    uint32_t crc_calculation;
    uint32_t crc_polynomial;
    uint32_t crc_length;
    uint32_t nssp_mode;
}__attribute__((__packed__))tctm_spi_init_s;

typedef struct _tctm_set_conv_board_par
{
    uint8_t mppt_conv1_enb;
    uint8_t mppt_conv2_enb;
    uint8_t mppt_conv3_enb;
    uint8_t mppt_conv4_enb;
    uint8_t op_conv1_enb;
    uint8_t op_conv2_enb;
    uint8_t op_conv3_enb;
    uint8_t op_conv4_enb;
    float conv1_op_vol_val;
    float conv2_op_vol_val;
    float conv3_op_vol_val;
    float conv4_op_vol_val;

}__attribute__((__packed__))tctm_set_conv_board_par_s;


typedef struct _tctm_get_board_par
{
   uint8_t mppt_conv1_enb;
   uint8_t mppt_conv2_enb;
   uint8_t mppt_conv3_enb;
   uint8_t mppt_conv4_enb;
   uint8_t op_conv1_enb;
   uint8_t op_conv2_enb;
   uint8_t op_conv3_enb;
   uint8_t op_conv4_enb;
   float conv1_op_vol_val;
   float conv2_op_vol_val;
   float conv3_op_vol_val;
   float conv4_op_vol_val;
   float pv1_vol;
   float pv2_vol;
   float pv3_vol;
   float pv4_vol;
   float pv1_cur;
   float pv2_cur;
   float pv3_cur;
   float pv4_cur;

}__attribute__((__packed__))tctm_get_conv_board_par_s;

typedef struct _tctm_set_batt_board_par
{
    float bat_cur_pos;
    float bat_cur_neg;
    uint8_t batt1_temp_low_lmt_val;
    uint8_t batt1_temp_upp_lmt_val;
    uint8_t batt2_temp_low_lmt_val;
    uint8_t batt2_temp_upp_lmt_val;
    uint8_t batt3_temp_low_lmt_val;
    uint8_t batt3_temp_upp_lmt_val;
    uint8_t batt4_temp_low_lmt_val;
    uint8_t batt4_temp_upp_lmt_val;
    uint8_t batt5_temp_low_lmt_val;
    uint8_t batt5_temp_upp_lmt_val;
    uint8_t batt6_temp_low_lmt_val;
    uint8_t batt6_temp_upp_lmt_val;
    uint8_t batt7_temp_low_lmt_val;
    uint8_t batt7_temp_upp_lmt_val;
    uint8_t batt8_temp_low_lmt_val;
    uint8_t batt8_temp_upp_lmt_val;

}__attribute__((__packed__))tctm_set_batt_board_par_s;

typedef struct _tctm_get_batt_board_par
{
   float bat_cur_pos;
   float bat_cur_neg;
   uint8_t batt1_temp_low_lmt_val;
   uint8_t batt1_temp_upp_lmt_val;
   uint8_t batt2_temp_low_lmt_val;
   uint8_t batt2_temp_upp_lmt_val;
   uint8_t batt3_temp_low_lmt_val;
   uint8_t batt3_temp_upp_lmt_val;
   uint8_t batt4_temp_low_lmt_val;
   uint8_t batt4_temp_upp_lmt_val;
   uint8_t batt5_temp_low_lmt_val;
   uint8_t batt5_temp_upp_lmt_val;
   uint8_t batt6_temp_low_lmt_val;
   uint8_t batt6_temp_upp_lmt_val;
   uint8_t batt7_temp_low_lmt_val;
   uint8_t batt7_temp_upp_lmt_val;
   uint8_t batt8_temp_low_lmt_val;
   uint8_t batt8_temp_upp_lmt_val;
   float tot_batt_vol;
   float batt1_vol;
   float batt2_vol;
   float batt3_vol;
   float batt_cur;
   uint8_t batt1_temp;
   uint8_t batt2_temp;
   uint8_t batt3_temp;
   uint8_t batt4_temp;
   uint8_t batt5_temp;
   uint8_t batt6_temp;
   uint8_t batt7_temp;
   uint8_t batt8_temp;

}__attribute__((__packed__))tctm_get_batt_board_par_s;

typedef struct _tctm_out_chnl_enb
{
    uint8_t ch1_enb;
    uint8_t ch2_enb;
    uint8_t ch3_enb;
    uint8_t ch4_enb;
    uint8_t ch5_enb;
    uint8_t ch6_enb;
    uint8_t ch7_enb;
    uint8_t ch8_enb;
    uint8_t ch9_enb;
    uint8_t ch10_enb;
    uint8_t ch11_enb;
    uint8_t ch12_enb;

}__attribute__((__packed__))tctm_out_chnl_enb_s;

typedef struct _tctm_out_chnl_cur_lmt 
{
    float ch1_12v_cur_val;
    float ch2_12v_cur_val;
    float ch3_5v_cur_val;
    float ch4_12v_cur_val;
    float ch4_3v_cur_val;
    float ch5_5v_cur_val;
    float ch5_3v_cur_val;
    float ch5_batt_v_cur_val;
    float ch6_12v_cur_val;
    float ch6_3v_cur_val;
    float ch7_3v_cur_val;
    float ch8_5v_cur_val;
    float ch8_3v_cur_val;
    float ch9_5v_cur_val;
    float ch10_batt_v_cur_val;
    float ch11_batt_v_cur_val;
    float ch12_batt_v_cur_val;

}__attribute__((__packed__))tctm_out_chnl_cur_lmt_s;

typedef struct _tctm_out_chnl_par
{
    uint8_t ch1_enb;
    uint8_t ch2_enb;
    uint8_t ch3_enb;
    uint8_t ch4_enb;
    uint8_t ch5_enb;
    uint8_t ch6_enb;
    uint8_t ch7_enb;
    uint8_t ch8_enb;
    uint8_t ch9_enb;
    uint8_t ch10_enb;
    uint8_t ch11_enb;
    uint8_t ch12_enb;
    float ch1_12v_cur_val;
    float ch2_12v_cur_val;
    float ch3_5v_cur_val;
    float ch4_12v_cur_val;
    float ch4_3v_cur_val;
    float ch5_5v_cur_val;
    float ch5_3v_cur_val;
    float ch5_batt_v_cur_val;
    float ch6_12v_cur_val;
    float ch6_3v_cur_val;
    float ch7_3v_cur_val;
    float ch8_5v_cur_val;
    float ch8_3v_cur_val;
    float ch9_5v_cur_val;
    float ch10_batt_v_cur_val;
    float ch11_batt_v_cur_val;
    float ch12_batt_v_cur_val;

}__attribute__((__packed__))tctm_get_out_chnl_par_s;

typedef struct _tctm_eps_mode_par
{
    float  nrml_mde_thrld_vol_val;
    float safe_mde_thrld_vol_val;
    float crtl_mde_thrld_vol_val;
    uint8_t full_mde_ch1_enb;
    uint8_t full_mde_ch2_enb;
    uint8_t full_mde_ch3_enb;
    uint8_t full_mde_ch4_enb;
    uint8_t full_mde_ch5_enb;
    uint8_t full_mde_ch6_enb;
    uint8_t full_mde_ch7_enb;
    uint8_t full_mde_ch8_enb;
    uint8_t full_mde_ch9_enb;
    uint8_t full_mde_ch10_enb;
    uint8_t full_mde_ch11_enb;
    uint8_t full_mde_ch12_enb;
    uint8_t nml_mde_ch1_enb;
    uint8_t nml_mde_ch2_enb;
    uint8_t nml_mde_ch3_enb;
    uint8_t nml_mde_ch4_enb;
    uint8_t nml_mde_ch5_enb;
    uint8_t nml_mde_ch6_enb;
    uint8_t nml_mde_ch7_enb;
    uint8_t nml_mde_ch8_enb;
    uint8_t nml_mde_ch9_enb;
    uint8_t nml_mde_ch10_enb;
    uint8_t nml_mde_ch11_enb;
    uint8_t nml_mde_ch12_enb;
    uint8_t safe_mde_ch1_enb;
    uint8_t safe_mde_ch2_enb;
    uint8_t safe_mde_ch3_enb;
    uint8_t safe_mde_ch4_enb;
    uint8_t safe_mde_ch5_enb;
    uint8_t safe_mde_ch6_enb;
    uint8_t safe_mde_ch7_enb;
    uint8_t safe_mde_ch8_enb;
    uint8_t safe_mde_ch9_enb;
    uint8_t safe_mde_ch10_enb;
    uint8_t safe_mde_ch11_enb;
    uint8_t safe_mde_ch12_enb;
    uint8_t crtl_mde_ch1_enb;
    uint8_t crtl_mde_ch2_enb;
    uint8_t crtl_mde_ch3_enb;
    uint8_t crtl_mde_ch4_enb;
    uint8_t crtl_mde_ch5_enb;
    uint8_t crtl_mde_ch6_enb;
    uint8_t crtl_mde_ch7_enb;
    uint8_t crtl_mde_ch8_enb;
    uint8_t crtl_mde_ch9_enb;
    uint8_t crtl_mde_ch10_enb;
    uint8_t crtl_mde_ch11_enb;
    uint8_t crtl_mde_ch12_enb;

}__attribute__((__packed__))tctm_eps_mode_par_s;

/**
 * @brief This API creates socket interface to send Telecommand data
 *
 * @param[in] params    : Unused parameter
 *
 * @return Result of API execution status
 * @retval 0 -> Success / other values -> Error.
 */
void *mfcc_usr_input_thread(void *params);

/**
 * @brief This API parses the received Telemetry information
 *
 * @param[in] tm_data   : contains the TM binary data stream
 *
 * @return Result of API execution status
 * @retval 0 -> Success / other values -> Error.
 */
int mfcc_tm_parser(uint8 *tm_data);

/**
 * @brief This API validates the sequence number of received TM data
 *
 * @param[in] actual_seq_no    : Sequence number of received TM data frame
 *
 * @return Result of API execution status
 * @retval 0 -> Success / other values -> Error.
 */
void mfcc_validate_seq_num(uint16 actual_seq_no);

void *mfcc_ft_tc_sender_thread (void * params);

int8 tc_trgt_point_req_enc(uint8* data, tctm_trgt_pt_req_s *tpr);
int8 tc_spi_intf_param_enc(uint8* data, tctm_spi_init_s *spii);
int8 tc_ethernet_init_param_enc(uint8* data, tctm_ethernet_init_s *eth);
int8 tc_can_init_param_enc(uint8* data, tctm_can_init_s *cis);
int8 tc_i2c_init_param_enc(uint8* data, tctm_i2c_init_s *i2ci);
int8 tc_usb_init_param_enc(uint8* data, tctm_usb_otg_cfg_s *usbo);
int8 tc_uart_init_param_enc(uint8* data, tctm_uart_init_s *uarti);
int8 tm_gnss_loc_info_dec(uint8* data, tctm_gnss_loc_info_s *gli);
int8 tm_spi_intf_param_dec(uint8* data, tctm_spi_init_s *spii);
int8 tm_ethernet_init_param_dec(uint8* data, tctm_ethernet_init_s *eth);
int8 tm_can_init_param_dec(uint8* data, tctm_can_init_s *cis);
int8 tm_i2c_init_param_dec(uint8* data, tctm_i2c_init_s *i2ci);
int8 tm_usb_init_param_dec(uint8* data, tctm_usb_otg_cfg_s *usbo);
int8 tm_uart_init_param_dec(uint8* data, tctm_uart_init_s *uarti);
void mfcc_tc_send(uint32 tc_id, uint16 da_id);

void mfcc_tc_send(uint32 tc_id, uint16 da_id);
int8 tc_set_conv_brd_par_enc(uint8_t* data,tctm_set_conv_board_par_s *brd);
int8 tc_set_batt_brd_par_enc(uint8_t* data,tctm_set_batt_board_par_s *batt);
int8 tc_out_chnl_enb_enc(uint8_t* data,tctm_out_chnl_enb_s *chnl);
int8 tc_out_chnl_cur_lmt_enc(uint8_t* data,tctm_out_chnl_cur_lmt_s *cur);
int8 tc_set_eps_mode_cfg_enc(uint8_t* data,tctm_eps_mode_par_s *eps);
int8 tc_get_conv_board_enc(uint8_t* data,tctm_get_conv_board_par_s *brd);
int8 tc_get_batt_board_enc(uint8_t* data,tctm_get_batt_board_par_s *batt);
int8 tc_get_out_chnl_par_enc(uint8_t* data,tctm_get_out_chnl_par_s *chnl);
int8 tc_get_eps_mode_enc(uint8_t* data,tctm_eps_mode_par_s *eps);
int8 tm_get_conv_brd_dec(uint8* data, tctm_get_conv_board_par_s *brd);
int8 tm_get_batt_brd_dec(uint8* data, tctm_get_batt_board_par_s *batt);
int8 tm_get_out_chnl_par_dec(uint8* data, tctm_get_out_chnl_par_s *chnl);
int8 tm_get_eps_mode_dec(uint8* data, tctm_eps_mode_par_s *eps);

int8 tm_tty_data_dec(uint8* data, uint8* tm_tty_data, uint32 data_size);
int8 tc_tty_data_enc(uint8* data, uint8* tc_tty_data, uint32 data_size);
int8 tc_schedule_cfg_lut_enc(uint8 *data, uint16 num_events);
int8 tc_schedule_deploy_enc(uint8 *data, uint8 num_days);

#endif
