/**
 * @file mfcc_tc_tm_sample.c
 *
 * @brief This file contains functions that provides demonstration of
 *        usage of TC TM Library APIs
 *
 * The functions provided in the file provides demonstration on
 * usage of TC TM library APIs
 */
/*********************************************************************/
#include "mfcc_tc_tm_sample.h"
#include <string.h>
#include <stdio.h>

#define SNS_GET_INF_PRINT
//#define ENBL_DBG_PRNT

#define MAX_CHUNK_SIZE 213
#define MAX_PAYLOAD_SIZE 4096
/*
 * added for FT
 */
#include "file_transfer.h"
#include "ft_sample_app_upload.h"

#include "mfcc_tc_tm_sample.h"
extern s_sock_info tctm_soc_inf;
/**
 * @brief Global variable to keep track of target pointing response status
 */
uint32 trgt_pt_rsp_sts = 0;
uint32 ps_pwr_on_rsp_sts = 0;
uint32 ps_pwr_off_rsp_sts = 0;
uint32 tty_start_rsp_sts = 0;
uint32 tty_end_rsp_sts = 0;
uint32 pld_hw_pwr_on_rsp_sts = 0;
uint32 pld_vm_pwr_on_rsp_sts = 0;
uint32 pld_hw_pwr_off_rsp_sts = 0;
uint32 pld_vm_pwr_off_rsp_sts = 0;
uint32 set_conv_brd_par = 0;
uint32 set_batt_brd_par = 0;
uint32 out_chnl_enb = 0;
uint32 out_chnl_cur_lmt = 0;
uint32 set_eps_mode = 0;

uint32 get_conv_brd_par = 0;
uint32 spi_intf_param_rsp_sts = 0;
uint32 eth_intf_param_rsp_sts = 0;
uint32 can_intf_param_rsp_sts = 0;
uint32 i2c_intf_param_rsp_sts = 0;
uint32 usb_intf_param_rsp_sts = 0;
uint32 uart_intf_param_rsp_sts = 0;
uint32 set_temp_sen_parm_sts =0;
uint32 set_imu_parm_sts = 0;
uint32 set_thrust_ctrl_parm_sts =0;
uint32 set_gpio_exp_parm_sts = 0;
uint32 set_dig_therm_parm_sts = 0;
uint32 set_pwr_mon_parm_sts = 0;
uint32 set_volt_ctrl_mon_parm_sts = 0;
uint32 set_pwr_seq_mon_parm_sts = 0;
uint32 set_data_log_parm_sts = 0;

temp_sens_param temp_sns_para;
temp_sens_info temp_sns_inf;
digital_thermostat_info dgtl_thrmst_inf;
digital_thermostat_param dgtl_thrmst_para;
hotswap_cntlr_info htswp_cntlr_inf;
hotswap_cntlr_param htswp_cntlr_para;
power_sense_mntr_info pwr_sns_mtr_inf;
power_sense_mntr_param pwr_sns_mtr_para;
voltage_seq_mntr_info vltg_seq_mntr_inf;
voltage_seq_mntr_param vltg_seq_mntr_para;
gpio_exp_info gpio_exp_inf;
gpio_exp_param gpio_exp_para;
thruster_info thrstr_inf;
thruster_param thrstr_para;
data_acq_dvc_info  data_acq_inf;
data_acq_dvc_param data_acq_para;
imu_info imu_inf;
imu_param imu_para;

//char* tty_dcdu = "TTY sample debug command data unit";
char* tty_dcdu = "pwd";

uint8 tty_drdu[1000];
/**
 * @brief Global variable to keep track of location information
 */
tctm_gnss_loc_info_s gli;
tctm_get_conv_board_par_s brd;
tctm_get_batt_board_par_s batt;
tctm_get_out_chnl_par_s chnl;
tctm_out_chnl_cur_lmt_s cur;
tctm_eps_mode_par_s eps;
tctm_spi_init_s spii;
tctm_ethernet_init_s eth;
tctm_can_init_s cis;
tctm_i2c_init_s i2ci;
tctm_usb_otg_cfg_s usbo;
tctm_uart_init_s uarti;



typedef struct ft_tctm_pld_tx
{
	uint8_t *payload_ptr;
	uint16_t payload_len;
	uint16_t tc_tm_id;
	uint8_t send_request;
	uint16_t dest_node;
}ft_tctm_payload_tx;

ft_tctm_payload_tx ft_pld_tx;


uint8_t ft_payload_transmitter(uint16 tc_tm_id, uint8_t *payload_ptr, uint16_t payload_len);
void ft_destination_set(uint16_t set);



/**
 * @brief Function that prepares Telecommand payload
 * based on user input
 */
void *mfcc_usr_input_thread(void *params)
{
    uint32 tst_id;
    uint32 tc_id;
    while(1)
    {
        sleep(1);
        printf("Enter Test Type ID\n");
        printf("Test Type ID 1 - Send TC to OBC\n");
        printf("Test Type ID 2 - Send TC to OBC-COMMS\n");
        printf("Test Type ID 3 - Send TC to PS\n");
        printf("Test Type ID 4 - Select File Transfer\n");
        printf("Test Type ID 5 - Send TC to OBC-ADCS\n");
        scanf(" %d", &tst_id);
        getchar();

        if(1 == tst_id)
        {
            /* Get Telecommand ID to be sent from Ground station to Satellite */
            printf("Enter Telecommand ID \n");
            scanf(" %d", &tc_id);
            mfcc_tc_send(tc_id, TC_DST_OBC);
        }
        else if(2 == tst_id)
        {
            /* Get Telecommand ID to be sent from Ground station to COMMS Module */
            printf("Enter Telecommand ID \n");
            scanf(" %d", &tc_id);
            mfcc_tc_send(tc_id, TC_DST_SBND_COMMS);
        }
        else if(3 == tst_id)
        {
            /* Get Telecommand ID to be sent from Ground station to Satellite */
            printf("Enter Telecommand ID \n");
            scanf(" %d", &tc_id);
            mfcc_tc_send(tc_id, TC_DST_PS);
        }
        else if(4 == tst_id)
        {
            printf("File Transfer Initiated...\n");
            ft_pld_tx.send_request = 0;

            printf("Enter Destination node\n");
            printf("Destination Type ID 1 -  Upload to OBC\n");
            printf("Destination Type ID 2 -  Upload to PS\n");
            printf("Destination Type ID 3 -  None of the above\n");

            scanf(" %d", &tst_id);
            getchar();
            if(1 == tst_id)
            {
            	ft_destination_set(TC_DST_OBC);
                printf("OBC selected\n");
            }
            else  if(2 == tst_id)
            {
            	ft_destination_set(TM_SRC_PS);
                printf("PS selected\n");
            }
            else
            {
            	ft_destination_set(TC_DST_PS);
                printf("Invalid selection ; ");
                printf("Default option PS selected\n");
            }

            ft_pld_tx.send_request = 0;
            ft_register_pld_trasnmitter_cblk(ft_payload_transmitter);
        	app1234_thread();
        }
        else if(5 == tst_id)
        {
            /* Get Telecommand ID to be sent from Ground station to Satellite */
            printf("Enter Telecommand ID \n");
            scanf(" %d", &tc_id);
            mfcc_tc_send(tc_id, TC_DST_ADCS);
        }
        else
        {
            printf("Invalid Test ID Received - %d. Please try again. \n",tst_id);
        }

    }
}

void mfcc_tc_send(uint32 tc_id, uint16 da_id)
{
    int32 port = atoi(tctm_soc_inf.port_num);
    int32 addr = inet_addr(tctm_soc_inf.ip_addr);
    s_tc_frame *tc_orig_app;
    uint8 *dst;
    int8 status = 0;
    uint16 tc_len;
    uint16 num_events;
    uint8 num_days;
    uint8 lut_id;
    int32 size = 0;
    uint8 *tc_data = NULL;
    tctm_trgt_pt_req_s tpr;
    uint32 pld_hw_id;
    uint32 pld_vm_id;
    tctm_set_conv_board_par_s board;
    tctm_set_batt_board_par_s batt;
    tctm_out_chnl_enb_s chnl;
    tctm_out_chnl_cur_lmt_s cur;
    tctm_eps_mode_par_s eps;
    s_comms_tx_cfg set_tx_cfg;
    s_comms_rx_cfg set_rx_cfg;
    s_comms_sys_cfg set_sys_cfg;
    uint32 loop = 0;

    /* Encryption related params */
    s_crypt_params param;
    uint8 key[] = {0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

    uint8 iv[16] = {0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                    0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

    /* Send buffer and frame memory allocations */
    tc_orig_app = (s_tc_frame *) malloc(sizeof(s_tc_frame));
    dst = (unsigned char *) malloc(MAX_TC_BUFF_SIZE);

    /* The above part need to be built in the Go Language */

    /* From here onwards TC TM library is used */
    /* Initialize telecommand structure instance */
    tc_orig_init(tc_orig_app);
    /* Reset the sequence number */
    //tc_orig_reset_seq_no();
    /* Configure Start of Frame */
    tc_orig_cfg_sof(tc_orig_app, TC_FRAME_SOF);
    /* Configure Satellite ID */
    tc_orig_cfg_sat_id(tc_orig_app, RECV_SAT_ID);
    /* Configure Ground station ID */
    tc_orig_cfg_gs_id(tc_orig_app, SOURCE_GROUND_STN_ID);
    /* Enable encryption */
    tc_orig_cfg_tc_ctrl(tc_orig_app, CRYPTO_ENCRYP_ENBL);
    /* Configure Quality of Service */
    tc_orig_cfg_qos(tc_orig_app, TC_QOS_CFG);
    /* Configure source address */
    tc_orig_cfg_sa_id(tc_orig_app, TC_SRC_GS);
    /* Configure destination address */
    tc_orig_cfg_da_id(tc_orig_app, da_id);

    switch(tc_id)
    {
        /* Telecommand prepartion for Target Pointing Request */
        case TC_TRGT_PT:
            {
                /* pack TC data */
                tc_len = sizeof(tpr);
                tc_data = (uint8_t *) malloc(tc_len+1);
                tpr.lat = 44;
                tpr.longit = 72;;
                tpr.alt = 101;
                tpr.pld_hw_id = 3;
                tpr.act_tmr_min = 10;
                tpr.act_tmr_sec = 0;
                tpr.act_tmr_ms = 0;
                tc_data[0] = TC_TRGT_PT;
                tc_trgt_point_req_enc(&tc_data[1], &tpr);
            }
            break;

        case TC_LOC_INFO:
            {
                /* pack TC payload */
                tc_len = 1;
                tc_data = (uint8_t *) malloc(tc_len);
                tc_data[0] = TC_LOC_INFO;
            }
            break;

        case TC_PS_PWR_ON:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_PLD_HW_PWR_ON:
            {
                /* pack TC payload */
                tc_len = sizeof(pld_hw_id);
                tc_data = (uint8_t *) malloc(tc_len);
                memcpy(tc_data,&pld_hw_id,tc_len);
            }
            break;

        case TC_PLD_VM_PWR_ON:
            {
                /* pack TC payload */
                tc_len = sizeof(pld_vm_id);
                tc_data = (uint8_t *) malloc(tc_len);
                memcpy(tc_data,&pld_vm_id,tc_len);
            }
            break;

        case TC_PLD_HW_PWR_OFF:
            {
                /* pack TC payload */
                tc_len = sizeof(pld_hw_id);
                tc_data = (uint8_t *) malloc(tc_len);
                memcpy(tc_data,&pld_hw_id,tc_len);
            }
            break;

        case TC_PLD_VM_PWR_OFF:
            {
                /* pack TC payload */
                tc_len = sizeof(pld_vm_id);
                tc_data = (uint8_t *) malloc(tc_len);
                memcpy(tc_data,&pld_vm_id,tc_len);
            }
            break;

        case TC_PS_PWR_OFF:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_TTY_START:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_TTY_END:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_TTY_DEBUG:
            {
                /* pack TC data */
                tc_len = strlen(tty_dcdu);
                tc_data = (uint8_t *) malloc(tc_len);
                tc_tty_data_enc(tc_data,(uint8*)tty_dcdu,tc_len);
            }
            break;

        case TC_SET_TEMP_SEN_PARM:
            {
                temp_sns_para.instance_id=1;
                temp_sns_para.high_temp_thrsld=70;
                temp_sns_para.low_temp_thrsld=30;
                temp_sns_para.critical_temp_thrsld=75;
                temp_sns_para.adc_resolution=3;
                temp_sns_para.operating_mode=1;
                temp_sns_para.polarity=1;
                temp_sns_para.hysteresis=0;
                temp_sns_para.window_lock=0;
                temp_sns_para.event_mode=0;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d\n",temp_sns_para.instance_id);
                printf("high_temp_thrsld = %d\n",temp_sns_para.high_temp_thrsld);
                printf("low_temp_thrsld = %d\n",temp_sns_para.low_temp_thrsld);
                printf("critical_temp_thrsld = %d\n",temp_sns_para.critical_temp_thrsld);
                printf("adc_resolution = %d\n",temp_sns_para.adc_resolution);
#endif

                /* pack TC data */
                tc_len = sizeof(temp_sens_param);
                tc_data = (uint8_t *) malloc(tc_len);

                /* encode */
                memcpy(tc_data,&temp_sns_para,tc_len);

            }
            break;

        case TC_GET_TEMP_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_SET_IMU_PARM:
            {
                imu_para.instance_id=2;
                imu_para.power=20;
                imu_para.odr=10;
                imu_para.range=8;
                imu_para.bw=14;

#ifdef SNS_GET_INF_PRINT

                printf("instance_id = %d \n",imu_para.instance_id);
                printf("power = %d \n",imu_para.power);
                printf("odr = %d \n",imu_para.odr);
                printf("bw = %d \n",imu_para.bw);
                printf("range = %d \n",imu_para.range);
#endif
                /* pack TC data */
                tc_len = sizeof(imu_param);
                tc_data = (uint8_t *) malloc(tc_len);

                /* encode */
                memcpy(tc_data,&imu_para,tc_len);
            }
            break;

        case TC_GET_IMU_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_SET_THRUST_CTRL_PARM:
            {
                thrstr_para.instance_id=3;
                thrstr_para.unit_num=2;
                thrstr_para.mode=0;
                thrstr_para.force=8.89;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",thrstr_para.instance_id);
                printf("unit_num = %d \n",thrstr_para.unit_num);
                printf("mode = %d \n",thrstr_para.mode);
                printf("force = %f \n",thrstr_para.force);
#endif
                /* pack TC data */
                tc_len = sizeof(thruster_param);
                tc_data = (uint8_t *) malloc(tc_len);
                /* encode */
                memcpy(tc_data,&thrstr_para,tc_len);
            }
            break;

        case TC_GET_THRUST_CTRL_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_SET_GPIO_EXP_PARM:
            {
                gpio_exp_para.instance_id=4;
                gpio_exp_para.intr_pin_polarity=0;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",gpio_exp_para.instance_id);
                printf("intr_pin_polarity = %d \n",gpio_exp_para.intr_pin_polarity);
#endif
                /* pack TC data */
                tc_len = sizeof(gpio_exp_param);
                tc_data = (uint8_t *) malloc(tc_len);
                /* encode */
                memcpy(tc_data,&gpio_exp_para,tc_len);

            }
            break;

        case TC_GET_GPIO_EXP_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_SET_DIG_THERM_PARM:
            {

                dgtl_thrmst_para.instance_id =5;
                dgtl_thrmst_para.high_temp_thrsld=74;
                dgtl_thrmst_para.low_temp_thrsld=32;
                dgtl_thrmst_para.po_pin_logic=1;
                dgtl_thrmst_para.adc_resolution=2;
                dgtl_thrmst_para.operating_mode=0;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id=%d \n",dgtl_thrmst_para.instance_id);
                printf("high_temp_thrsld=%d\n",dgtl_thrmst_para.high_temp_thrsld);
                printf("low_temp_thrsld=%d\n",dgtl_thrmst_para.low_temp_thrsld);
                printf("po_pin_logic=%d\n",dgtl_thrmst_para.po_pin_logic);
                printf("adc_resolution=%d\n",dgtl_thrmst_para.adc_resolution);
#endif

                /* pack TC data */
                tc_len = sizeof(digital_thermostat_param);
                tc_data = (uint8_t *) malloc(tc_len);
                /* encode */
                memcpy(tc_data,&dgtl_thrmst_para,tc_len);
            }
            break;

        case TC_GET_DIG_THERM_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_SET_PWR_MON_PARM:
            {
                pwr_sns_mtr_para.instance_id=6;
                pwr_sns_mtr_para.vbus_thrsld=15;
                pwr_sns_mtr_para.vshunt_thrsld=10;
                pwr_sns_mtr_para.calibration=11;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",pwr_sns_mtr_para.instance_id);
                printf("vbus_thrsld = %d \n",pwr_sns_mtr_para.vbus_thrsld);
                printf("vshunt_thrsld = %d \n",pwr_sns_mtr_para.vshunt_thrsld);
                printf("calibration = %d \n",pwr_sns_mtr_para.calibration);
#endif
                /* pack TC data */
                tc_len = sizeof(power_sense_mntr_param);
                tc_data = (uint8_t *) malloc(tc_len);

                /* encode */
                memcpy(tc_data,&pwr_sns_mtr_para,tc_len);
            }
            break;
        case TC_GET_PWR_MON_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;
        case TC_SET_VOLT_CTRL_MON_PARM:
            {
                /* pack TC data */

                htswp_cntlr_para.instance_id=7;
                htswp_cntlr_para.hotswap_sts=1;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",htswp_cntlr_para.instance_id);
                printf("hotswap_sts = %d \n",htswp_cntlr_para.hotswap_sts);
#endif

                tc_len = sizeof(hotswap_cntlr_param);
                tc_data = (uint8_t *) malloc(tc_len);

                /* encode */
                memcpy(tc_data,&htswp_cntlr_para,tc_len);
            }
            break;

        case TC_GET_VOLT_CTRL_MON_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_SET_PWR_SEQ_MON_PARM:
            {
                vltg_seq_mntr_para.instance_id=14;
                vltg_seq_mntr_para.v_ref_type=0;
                vltg_seq_mntr_para.flash_lock_sts=1;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",vltg_seq_mntr_para.instance_id);
                printf("flash_lock_sts = %d \n",vltg_seq_mntr_para.flash_lock_sts);
                printf("v_ref_type = %d \n",vltg_seq_mntr_para.v_ref_type);
#endif

                /* pack TC data */
                tc_len = sizeof(voltage_seq_mntr_param);
                tc_data = (uint8_t *) malloc(tc_len);

                /* encode */
                memcpy(tc_data,&vltg_seq_mntr_para,tc_len);
            }
            break;
        case TC_GET_PWR_SEQ_MON_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;
        case TC_SET_DATA_LOG_PARM:
            {
                data_acq_para.instance_id=12;
                data_acq_para.chnl_num=5;
                data_acq_para.pwr_dwn_optn=1;

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",data_acq_para.instance_id);
                printf("chnl_num = %d \n",data_acq_para.chnl_num);
                printf("pwr_dwn_optn = %d \n",data_acq_para.pwr_dwn_optn);
#endif

                /* pack TC data */
                tc_len = sizeof(data_acq_dvc_param);
                tc_data = (uint8_t *) malloc(tc_len);

                /* encode */
                memcpy(tc_data,&data_acq_para,tc_len);
            }
            break;

        case TC_GET_DATA_LOG_INFO:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_SET_SPI_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = sizeof(spii);
                tc_data = (uint8_t *) malloc(tc_len);
                spii.mode = 12;
                spii.direction = 0;
                spii.datasize = 33;
                spii.clk_polarity = 34;
                spii.clk_phase = 35;
                spii.nss = 31;
                spii.baudrate_prescaler = 76;
                spii.firstbit = 32;
                spii.timode = 14;
                spii.crc_calculation = 15;
                spii.crc_polynomial = 16;
                spii.crc_length = 14;
                spii.nssp_mode = 11;
                tc_spi_intf_param_enc(tc_data, &spii);


            }
            break;
        case TC_SET_ETH_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = sizeof(eth);
                tc_data = (uint8_t *) malloc(tc_len);
                eth.auto_negotiation = 0;
                eth.speed = 89;
                eth.duplex_mode = 34;
                eth.phy_address = 23;
                eth.mac_addr = (uint8*)21;
                eth.rx_mode = 21;
                eth.checksum_mode = 20;
                eth.media_interface = 10;
                tc_ethernet_init_param_enc(tc_data, &eth);


            }
            break;
        case TC_SET_CAN_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = sizeof(cis);
                tc_data = (uint8_t *) malloc(tc_len);
                cis.prescaler = 1;
                cis.mode = 4;
                cis.syncjumpwidth = 56;
                cis.time_seg1 = 23;
                cis.time_seg2 = 22;
                cis.time_triggered_mode = 2;
                cis.autobus_off = 0;
                cis.autowake_up = 90;
                cis.auto_retransmission = 12;
                cis.receive_fifo_locked = 21;
                cis.transmit_fifo_priority = 11;
                tc_can_init_param_enc(tc_data, &cis);


            }
            break;
        case TC_SET_I2C_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = sizeof(i2ci);
                tc_data = (uint8_t *) malloc(tc_len);
                i2ci.timing = 100;
                i2ci.own_address1 = 24;
                i2ci.addressing_mode = 26;
                i2ci.dual_address_mode = 77;
                i2ci.own_address2 = 9;
                i2ci.own_address2_masks = 10;
                i2ci.generalcall_mode = 21;
                i2ci.no_stretch_mode = 20;
                tc_i2c_init_param_enc(tc_data, &i2ci);


            }
            break;
        case TC_SET_USB_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = sizeof(usbo);
                tc_data = (uint8_t *) malloc(tc_len);
                usbo.dev_endpoints = 8;
                usbo.host_channels = 6;
                usbo.speed = 2;
                usbo.dma_enable = 1;
                usbo.ep0_mps = 23;
                usbo.phy_itface = 10;
                usbo.sof_enable = 13;
                usbo.low_power_enable = 34;
                usbo.lpm_enable = 22;
                usbo.battery_charging_enable = 90;
                usbo.vbus_sensing_enable = 80;
                usbo.use_dedicated_ep1 = 34;
                usbo.use_external_vbus = 35;
                tc_usb_init_param_enc(tc_data, &usbo);


            }
            break;

        case TC_SET_UART_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = sizeof(uarti);
                tc_data = (uint8_t *) malloc(tc_len);
                uarti.baudrate = 34;
                uarti.wordlength = 1;
                uarti.stopbits = 0;
                uarti.parity = 2;
                uarti.mode = 5;
                uarti.hw_flow_ctl = 6;
                uarti.over_sampling = 9;
                uarti.onebit_sampling = 11;
                tc_uart_init_param_enc(tc_data, &uarti);


            }
            break;
        case TC_GET_SPI_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_GET_ETH_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);
            }
            break;

        case TC_GET_CAN_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);


            }
            break;

        case TC_GET_I2C_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);


            }
            break;

        case TC_GET_USB_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);


            }
            break;

        case TC_GET_UART_INTF_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_SCHEDULE_CFG_LUT:
            {
                num_events = 23;
                tc_len = (sizeof(lut_id) + sizeof(num_events) + (num_events * sizeof(schedule_event_s)));
                tc_data = (uint8 *) malloc(tc_len);
                tc_schedule_cfg_lut_enc(tc_data, num_events);
            }
            break;

        case TC_SCHEDULE_DEPLOY:
            {
                num_days = 2;
                tc_len = (sizeof(num_days) + (num_days * sizeof(orbit_lut_cfg_s)));
                tc_data = (uint8 *) malloc(tc_len);
                tc_schedule_deploy_enc(tc_data, num_days);
            }
            break;

        case TC_SET_CONV_BOARD_PARM:
            {
                tc_len = sizeof(tctm_set_conv_board_par_s);
                tc_data = (uint8_t *) malloc(tc_len);
                board.mppt_conv1_enb=1;
                board.mppt_conv2_enb=1;
                board.mppt_conv3_enb=1;
                board.mppt_conv4_enb=1;
                board.op_conv1_enb=1;
                board.op_conv2_enb=1;
                board.op_conv3_enb=1;
                board.op_conv4_enb=1;
                board.conv1_op_vol_val=12;
                board.conv2_op_vol_val=3.3;
                board.conv3_op_vol_val=5;
                board.conv4_op_vol_val=12;
                tc_set_conv_brd_par_enc(tc_data, &board);

            }
            break;

        case TC_SET_BATT_BOARD_PARM:
            {
                tc_len = sizeof(tctm_get_batt_board_par_s);
                tc_data = (uint8_t *) malloc(tc_len);
                batt.bat_cur_pos=10;
                batt.bat_cur_neg=20;
                batt.batt1_temp_low_lmt_val=15;
                batt.batt1_temp_upp_lmt_val=25;
                batt.batt2_temp_low_lmt_val=15;
                batt.batt2_temp_upp_lmt_val=25;
                batt.batt3_temp_low_lmt_val=15;
                batt.batt3_temp_upp_lmt_val=25;
                batt.batt4_temp_low_lmt_val=15;
                batt.batt4_temp_upp_lmt_val=25;
                batt.batt5_temp_low_lmt_val=15;
                batt.batt5_temp_low_lmt_val=25;
                batt.batt6_temp_low_lmt_val=15;
                batt.batt6_temp_low_lmt_val=25;
                batt.batt7_temp_low_lmt_val=15;
                batt.batt7_temp_low_lmt_val=25;
                tc_set_batt_brd_par_enc(tc_data, &batt);

            }
            break;

        case TC_SET_OUT_CHNL_ENB:
            {
                tc_len = sizeof(tctm_out_chnl_enb_s);
                tc_data = (uint8_t *) malloc(tc_len);
                chnl.ch1_enb=1;
                chnl.ch2_enb=1;
                chnl.ch3_enb=1;
                chnl.ch4_enb=1;
                chnl.ch5_enb=1;
                chnl.ch6_enb=1;
                chnl.ch7_enb=1;
                chnl.ch8_enb=1;
                chnl.ch9_enb=1;
                chnl.ch10_enb=1;
                chnl.ch11_enb=1;
                chnl.ch12_enb=1;
                tc_out_chnl_enb_enc(tc_data, &chnl);
            }
            break;

        case TC_SET_OUT_CHNL_CUR_LMT:
            {
                tc_len = sizeof(tctm_out_chnl_cur_lmt_s);
                tc_data = (uint8_t *) malloc(tc_len);
                cur.ch1_12v_cur_val=0.84;
                cur.ch2_12v_cur_val=1.25;
                cur.ch3_5v_cur_val=1;
                cur.ch4_12v_cur_val=3;
                cur.ch4_3v_cur_val=1;
                cur.ch5_5v_cur_val=1;
                cur.ch5_3v_cur_val=1;
                cur.ch5_batt_v_cur_val=2;
                cur.ch6_12v_cur_val=3;
                cur.ch6_3v_cur_val=0.3;
                cur.ch7_3v_cur_val=0.6;
                cur.ch8_5v_cur_val=1;
                cur.ch8_3v_cur_val=1;
                cur.ch9_5v_cur_val=1;
                cur.ch10_batt_v_cur_val=10;
                cur.ch11_batt_v_cur_val=10;
                cur.ch12_batt_v_cur_val=5;
                tc_out_chnl_cur_lmt_enc(tc_data, &cur);
            }
            break;

        case TC_SET_EPS_MODE_CFG:
            {
                tc_len = sizeof(tctm_eps_mode_par_s);
                tc_data = (uint8_t *) malloc(tc_len);
                eps.nrml_mde_thrld_vol_val=11.7;
                eps.safe_mde_thrld_vol_val=10.8;
                eps.crtl_mde_thrld_vol_val=9.9;
                eps.full_mde_ch1_enb=1;
                eps.full_mde_ch2_enb=1;
                eps.full_mde_ch3_enb=1;
                eps.full_mde_ch4_enb=1;
                eps.full_mde_ch5_enb=1;
                eps.full_mde_ch6_enb=1;
                eps.full_mde_ch7_enb=1;
                eps.full_mde_ch8_enb=1;
                eps.full_mde_ch9_enb=1;
                eps.full_mde_ch10_enb=1;
                eps.full_mde_ch11_enb=1;
                eps.full_mde_ch12_enb=1;
                eps.nml_mde_ch1_enb=1;
                eps.nml_mde_ch2_enb=1;
                eps.nml_mde_ch3_enb=1;
                eps.nml_mde_ch4_enb=1;
                eps.nml_mde_ch5_enb=1;
                eps.nml_mde_ch6_enb=1;
                eps.nml_mde_ch7_enb=1;
                eps.nml_mde_ch8_enb=1;
                eps.nml_mde_ch9_enb=1;
                eps.nml_mde_ch10_enb=1;
                eps.nml_mde_ch11_enb=1;
                eps.nml_mde_ch12_enb=1;
                eps.safe_mde_ch1_enb=1;
                eps.safe_mde_ch2_enb=1;
                eps.safe_mde_ch3_enb=1;
                eps.safe_mde_ch4_enb=1;
                eps.safe_mde_ch5_enb=1;
                eps.safe_mde_ch6_enb=1;
                eps.safe_mde_ch7_enb=1;
                eps.safe_mde_ch8_enb=1;
                eps.safe_mde_ch9_enb=1;
                eps.safe_mde_ch10_enb=1;
                eps.safe_mde_ch11_enb=1;
                eps.safe_mde_ch12_enb=1;
                eps.crtl_mde_ch1_enb=1;
                eps.crtl_mde_ch2_enb=1;
                eps.crtl_mde_ch3_enb=1;
                eps.crtl_mde_ch4_enb=1;
                eps.crtl_mde_ch5_enb=1;
                eps.crtl_mde_ch6_enb=1;
                eps.crtl_mde_ch7_enb=1;
                eps.crtl_mde_ch8_enb=1;
                eps.crtl_mde_ch9_enb=1;
                eps.crtl_mde_ch10_enb=1;
                eps.crtl_mde_ch11_enb=1;
                eps.crtl_mde_ch12_enb=1;
                tc_set_eps_mode_cfg_enc(tc_data, &eps);
            }
            break;


        case TC_GET_CONV_BOARD_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_GET_BATT_BOARD_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_GET_OUT_CHNL_PARM:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;

        case TC_GET_EPS_MODE_CFG:
            {
                /* pack TC payload */
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

            }
            break;


		case TC_FTCI:
		case TC_FTFCI:
		case TC_FTDS:
		case TC_FTSR:
			{

                status = 1;
                printf("sent FT TC ..  \n");


				/* Configure Telecommand ID */
				tc_orig_cfg_tc_id(tc_orig_app, tc_id);


				/* Configure Telecommand Length */
				tc_orig_cfg_tc_len(tc_orig_app, ft_pld_tx.payload_len);
				/* Configure Telecommand payload */
				tc_orig_cfg_tc_data (tc_orig_app, ft_pld_tx.payload_ptr);

				/* Configure Encryption parameters */
				tc_orig_app->cmac_hmac = (uint8_t *) malloc(SHA256_HASH_SIZE);
				memcpy(param.iv, iv, 16);
				param.key_size = 256;
				param.key = (unsigned char *)malloc(param.key_size);
				memcpy(param.key, key, param.key_size);
				param.encrypt_mode = CRYPT_AES_CFB8;
				tc_orig_select_encryption(&param);

				/* Generate Telecommand as binary data stream */
				size = tc_pack(dst, tc_orig_app);
				/* Send generated Telecommand via socket interface */
				mfcc_tcp_send(addr, port, dst, size);
				free(ft_pld_tx.payload_ptr);
			}
break;


            /* Telecommand prepartion to set comms TX config */
        case TC_SET_COMMS_TX_CFG:
            {
                /* pack TC data */
                tc_len = sizeof(s_comms_tx_cfg);
                tc_data = (uint8_t *) malloc(tc_len);

                printf("Enter Transmit frequency (Type: uint32)\n");
                scanf(" %d", &set_tx_cfg.tx_cfg_freq);

                printf("Enter Transmit modulation type (Type: 0,1,2)\n");
                scanf(" %hhd", &set_tx_cfg.tx_cfg_type);

                printf("Enter Transmit symbol rate (Type: uint32)\n");
                scanf(" %d", &set_tx_cfg.tx_cfg_rate);

                printf("Enter RRC roll-off (Type: 0, 1, 2)\n");
                scanf(" %hhd", &set_tx_cfg.tx_cfg_rolloff);

                os_memcpy(tc_data, &set_tx_cfg, sizeof(s_comms_tx_cfg));

                status = 0;
            }
            break;

            /* Telecommand prepartion to set comms RX config */
        case TC_SET_COMMS_RX_CFG:
            {
                /* pack TC data */
                tc_len = sizeof(s_comms_rx_cfg);
                tc_data = (uint8_t *) malloc(tc_len);

                printf("Enter Receive frequency (Type: uint32)\n");
                scanf(" %d", &set_rx_cfg.rx_cfg_freq);

                printf("Enter Receive modulation type (Type: 0,1,2)\n");
                scanf(" %hhd", &set_rx_cfg.rx_cfg_type);

                printf("Enter Receive symbol rate (Type: uint32)\n");
                scanf(" %d", &set_rx_cfg.rx_cfg_rate);

                printf("Enter RRC roll-off (Type: 0, 1, 2)\n");
                scanf(" %hhd", &set_rx_cfg.rx_cfg_rolloff);

                os_memcpy(tc_data, &set_rx_cfg, sizeof(s_comms_rx_cfg));
                status = 0;

            }
            break;

            /* Telecommand prepartion to set comms SYS config */
        case TC_SET_COMMS_SYS_CFG:
            {
                /* pack TC data */
                tc_len = sizeof(s_comms_sys_cfg);
                tc_data = (uint8_t *) malloc(tc_len);

                printf("Enter system String name (Type: Max 20 characters)\n");
                scanf(" %s", set_sys_cfg.sys_name);

                printf("Enter Serial number/ID (Type: uint32)\n");
                scanf(" %d", &set_sys_cfg.sys_serial);

                printf("Enter Running software version (Type: uint32)\n");
                scanf(" %d", &set_sys_cfg.sys_sw_version);

                printf("Enter sys_bootcount (Type: uint32)\n");
                scanf(" %d", &set_sys_cfg.sys_bootcount);

                os_memcpy(tc_data, &set_sys_cfg, sizeof(s_comms_sys_cfg));
                status = 0;
            }
            break;

        case TC_GET_COMMS_TX_CFG:
        case TC_GET_COMMS_RX_CFG:
        case TC_GET_COMMS_SYS_CFG:
            {
                tc_len = 0;
                tc_data = (uint8_t *) malloc(tc_len);

                printf("In telemetry request case \n");
                status = 0;
                break;
            }

        default :
            {
                status = -1;
                printf("Invalid TC ID Received - %d. Please try again. \n",tc_id);
            }
    }
    if(0 == status)
    {

        /* Configure Telecommand ID */
        tc_orig_cfg_tc_id(tc_orig_app, tc_id);
        /* Configure Telecommand Length */
        tc_orig_cfg_tc_len(tc_orig_app, tc_len);
        /* Configure Telecommand payload */
        tc_orig_cfg_tc_data (tc_orig_app, tc_data);

        /* Configure Encryption parameters */
        tc_orig_app->cmac_hmac = (uint8_t *) malloc(SHA256_HASH_SIZE);
        memcpy(param.iv, iv, 16);
        param.key_size = 256;
        param.key = (unsigned char *)malloc(param.key_size);
        memcpy(param.key, key, param.key_size);
        param.encrypt_mode = CRYPT_AES_CFB8;
        tc_orig_select_encryption(&param);


        printf("MFCC APP: TC payload : ");
        for(loop = 0; loop < tc_len; loop++)
            printf("%x ", tc_data[loop]);
        printf("\n");
        /* Generate Telecommand as binary data stream */
        size = tc_pack(dst, tc_orig_app);

#ifdef ENBL_DBG_PRNT
        printf("MFCC APP: TC payload encrypted : ");
        for(loop = 0; loop < tc_len; loop++)
            printf("%x ", tc_data[loop]);
        printf("\n");
#endif

        printf("MFCC APP: Transmitted TC byte stream: ");
        for(loop = 0; loop < size; loop++)
            printf("%x ", dst[loop]);
        printf("\n");

        /* Send generated Telecommand via socket interface */
        mfcc_tcp_send(addr, port, dst, size);
#if 0
        memcpy(&buffer[0],&size,sizeof(uint16));
        memcpy(&buffer[2],dst,size);
        int itr=0, i=0;
        //printf("\nBUFFER COPIED\n");
        while(itr < size+2 )
        {
            printf("MFCC APP: Transmitted TC byte stream: ");
            for(loop = 0; loop < size+2; loop++)
                printf("%x ", buffer[loop]);
            printf("\n");

            /* Send generated Telecommand via socket interface */
            mfcc_tcp_send(addr, port, &buffer[MAX_CHUNK_SIZE * i], size+2);
            itr += MAX_CHUNK_SIZE;
            i++;
        }
#endif
    }
}



uint8_t ft_payload_transmitter(uint16 tc_tm_id, uint8_t *payload_ptr, uint16_t payload_len)
{
	uint8_t ret_busy_sts = 0;
	uint16_t u16cnt;
	if(payload_ptr != NULL)
	{
		if(ft_pld_tx.send_request != 1)
		{
			ft_pld_tx.payload_ptr = malloc(payload_len);
			if(ft_pld_tx.payload_ptr != NULL)
			{
				for(u16cnt = 0; u16cnt< payload_len; u16cnt++)
				{
					ft_pld_tx.payload_ptr[u16cnt] = payload_ptr[u16cnt];
				}
				ft_pld_tx.tc_tm_id = tc_tm_id;
				ft_pld_tx.payload_len = payload_len;
				ft_pld_tx.send_request = 1;
				ret_busy_sts = 1;
			}
		}
	}
	return ret_busy_sts;
}



void ft_destination_set(uint16_t set)
{
	ft_pld_tx.dest_node = set;
    printf("Node set %x\n",ft_pld_tx.dest_node);
}




void *mfcc_ft_tc_sender_thread(void *params)
{
    sleep(3);

    while(1)
    {
        if(ft_pld_tx.send_request ==1)
        {
        	printf("TC ID:%d to send %x\n",ft_pld_tx.tc_tm_id,ft_pld_tx.dest_node);

            mfcc_tc_send(ft_pld_tx.tc_tm_id, ft_pld_tx.dest_node);
            ft_pld_tx.send_request = 0;
        }
    }

}

/**
 * @brief Function that parses Telemetry payload
 * and invokes corresponding Telemetry decoder
 */
int mfcc_tm_parser(uint8 *tm_stream_data)
{
    s_tm_frame tm_frame;
    s_crypt_params param;
    s_tm_ctrl ctrl_param;
    uint16 tm_len = 0;
    uint16 seq_no = 0;
    uint16 tm_src_id, tm_dst_id;
    uint16 tm_id = 0;
    uint8 sts;
    s_comms_tx_cfg get_tx_cfg;
    s_comms_rx_cfg get_rx_cfg;
    s_comms_sys_cfg get_sys_cfg;
    int loop = 0;

    /* Define decryption key */
    uint8 key[] = {0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                   0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                   0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

    /* Define IV parameter for AES decryption */
    uint8 iv[16] = {0xff, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
        0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

    /* Allocation of memory for TM data */
    tm_frame.cmac_hmac = (uint8 *) malloc(SHA256_HASH_SIZE);
    tm_frame.tm_data = (uint8 *) malloc(MAX_TM_BUFF_SIZE);

    /* Unpack TM from binary stream of bytes */
    tm_unpack(tm_stream_data,&tm_frame);

    /* Decode TM header and payload information */
    if(NULL != tm_frame.tm_data)
    {
        /* Extract TM sequence number */
        tm_term_extract_seq_no(&tm_frame, &seq_no);
        /* Validate sequence number */
        mfcc_validate_seq_num(seq_no);
        /* Extract TM Source ID */
        tm_term_extract_sa_id(&tm_frame, &tm_src_id);
        /* Extract TM Destination ID */
        tm_term_extract_da_id(&tm_frame, &tm_dst_id);
        if(TM_SRC_OBC == tm_dst_id)
            printf("MFCC APP: Received TM Data from OBC\n");
        else if(TM_SRC_PS == tm_dst_id)
            printf("MFCC APP: Received TM Data from PS\n");
        else
        {}
        /* Extract Telemetry ID */
        tm_term_extract_tm_id(&tm_frame, &tm_id);
        /* Extract Encryption configuration */
        tm_term_extract_tm_ctrl(&tm_frame, &ctrl_param);
        /* When encryption is enabled in received TM */
        if(ctrl_param.encrypt)
        {
            tm_term_extract_tm_len(&tm_frame, &tm_len);

            /* Configure decryption parameters */
            memcpy(param.iv, iv, 16);
            param.key_size = 256;
            param.key = (unsigned char *)malloc(param.key_size);
            memcpy(param.key, key, param.key_size);
            param.encrypt_mode = CRYPT_AES_CFB8;
            /* Extract TM payload length */
            tm_term_extract_tm_len(&tm_frame, &tm_len);

            /* Decrypt Telemetry data */
            tm_orig_decrypt_data(&param,
                    (const unsigned char *)tm_frame.tm_data,tm_len,
                    (unsigned char *)tm_frame.tm_data);

            printf("MFCC APP: Decrypted TM data is ");
            for(loop = 0; loop < tm_len; loop++)
                printf("%x ", tm_frame.tm_data[loop]);
            printf("\n");
        }
        else
        {
            /* Extract TM payload length */
            tm_term_extract_tm_len(&tm_frame, &tm_len);
#ifdef ENBL_DBG_PRNT
            printf("MFCC APP: Received Data is unencrypted \n");
            printf("MFCC APP: Received Data length is %d\n",tm_len);
            printf("MFCC APP: Data is ");
            for(loop = 0; loop < tm_len; loop++)
                printf("%x ", tm_frame.tm_data[loop]);
            printf("\n");
#endif
        }
    }

    /* Decode TM payload information */
    switch(tm_id)
    {
        case TM_TRGT_PT:
            {
                memcpy(&trgt_pt_rsp_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!trgt_pt_rsp_sts)
                    printf("Target point configuration success\n");
                else
                    printf("Target point configuration failure\n");

            }
            break;

        case TM_LOC_INFO:
            {
                /* Decode Location information */
                tm_gnss_loc_info_dec(tm_frame.tm_data, &gli);
            }
            break;

        case TM_PS_PWR_ON:
            {
                memcpy(&ps_pwr_on_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!ps_pwr_on_rsp_sts)
                    printf("PS Power ON request success\n");
                else
                    printf("PS Power ON request failure\n");
            }
            break;
        case TM_PLD_HW_PWR_ON:
            {
                memcpy(&pld_hw_pwr_on_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!pld_hw_pwr_on_rsp_sts)
                    printf("PLD HW Power ON request success\n");
                else
                    printf("PLD HW Power ON request failure\n");
            }
            break;

        case TM_PLD_VM_PWR_ON:
            {
                memcpy(&pld_vm_pwr_on_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!pld_vm_pwr_on_rsp_sts)
                    printf("PLD VM Power ON request success\n");
                else
                    printf("PLD VM Power ON request failure\n");
            }
            break;

        case TM_PLD_HW_PWR_OFF:
            {
                memcpy(&pld_hw_pwr_off_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!pld_hw_pwr_off_rsp_sts)
                    printf("PLD HW Power OFF request success\n");
                else
                    printf("PLD HW Power OFF request failure\n");
            }
            break;

        case TM_PLD_VM_PWR_OFF:
            {
                memcpy(&pld_vm_pwr_off_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!pld_vm_pwr_off_rsp_sts)
                    printf("PLD VM Power OFF request success\n");
                else
                    printf("PLD VM Power OFF request failure\n");
            }
            break;

        case TM_PS_PWR_OFF:
            {
                memcpy(&ps_pwr_off_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!ps_pwr_on_rsp_sts)
                    printf("PS Power OFF request success\n");
                else
                    printf("PS Power OFF request failure\n");
            }
            break;

        case TM_SCHEDULE_CFG_LUT:
            {
                memcpy(&sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!sts)
                    printf("TC Schedule LUT Configuration success\n");
                else
                    printf("TC Schedule LUT Configuration failure\n");
            }
            break;

        case TM_SCHEDULE_DEPLOY:
            {
                memcpy(&sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!sts)
                    printf("TC Schedule Deploy success\n");
                else
                    printf("TC Schedule Deploy failure\n");
            }
            break;

        case TM_TTY_START:
            {
                memcpy(&tty_start_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!tty_start_rsp_sts)
                    printf("TTY start request success\n");
                else
                    printf("TTY start request failure\n");
            }
            break;

        case TM_TTY_END:
            {
                memcpy(&tty_end_rsp_sts,tm_frame.tm_data,4);
                /* Check PS power ON response status */
                if(!tty_end_rsp_sts)
                    printf("TTY end request success\n");
                else
                    printf("TTY end request failure\n");
            }
            break;

        case TM_TTY_DEBUG:
            {
                tm_tty_data_dec(tm_frame.tm_data,tty_drdu,tm_frame.tm_len);
                printf("TTY Debug Response data:\n%s\n",(char*)tty_drdu);
            }
            break;

        case TM_SET_TEMP_SEN_PARM:
            {
                memcpy(&set_temp_sen_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_temp_sen_parm_sts)
                    printf("Set temperature sensor parameter success\n");
                else
                    printf("Set temperature sensor parameter failure\n");

            }
            break;

        case TM_GET_TEMP_INFO:
            {
                /* Decode Location information */
                memcpy(&temp_sns_inf,tm_frame.tm_data, sizeof(temp_sens_info));

#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d\n",temp_sns_inf.instance_id);
                printf("high_temp_thrsld = %d\n",temp_sns_inf.high_temp_thrsld);
                printf("low_temp_thrsld = %d\n",temp_sns_inf.low_temp_thrsld);
                printf("critical_temp_thrsld = %d\n",temp_sns_inf.critical_temp_thrsld);
                printf("adc_resolution = %d\n",temp_sns_inf.adc_resolution);
                printf("dvc_id = %d\n",temp_sns_inf.dvc_id);
                printf("manufacture_id = %d\n",temp_sns_inf.manufacture_id);
#endif
            }
            break;

        case TM_SET_IMU_PARM:
            {
                memcpy(&set_imu_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_imu_parm_sts)
                    printf("Set IMU sensor parameter success\n");
                else
                    printf("Set IMU sensor parameter failure\n");

            }
            break;

        case TM_GET_IMU_INFO:
            {
                /* Decode Location information */
                memcpy(&imu_inf,tm_frame.tm_data, sizeof(imu_info));
#ifdef SNS_GET_INF_PRINT

                printf("instance_id = %d \n",imu_inf.instance_id);
                printf("power = %d \n",imu_inf.power);
                printf("accel_x = %d \n",imu_inf.accel_x);
                printf("gyro_y = %d \n",imu_inf.gyro_y);
                printf("mag_y = %d \n",imu_inf.mag_y);

#endif
            }
            break;

        case TM_SET_THRUST_CTRL_PARM:
            {
                memcpy(&set_thrust_ctrl_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_thrust_ctrl_parm_sts)
                    printf("Set thruster parameter success\n");
                else
                    printf("Set thruster parameter failure\n");

            }
            break;

        case TM_GET_THRUST_CTRL_INFO:
            {
                /* Decode Location information */
                memcpy(&thrstr_inf,tm_frame.tm_data, sizeof(thruster_info));
#ifdef SNS_GET_INF_PRINT

                printf("instance_id = %d \n",thrstr_inf.instance_id);
                printf("brd_err_count = %d \n",thrstr_inf.brd_err_count);
                printf("strt_tm = %d \n",thrstr_inf.strt_tm);
                printf("master_tm = %d \n",thrstr_inf.master_tm);
                printf("bkpln_master_tm = %d \n",thrstr_inf.bkpln_master_tm);

#endif
            }
            break;

        case TM_SET_GPIO_EXP_PARM:
            {
                memcpy(&set_gpio_exp_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_gpio_exp_parm_sts)
                    printf("Set gpio expander parameter success\n");
                else
                    printf("Set gpio expander parameter failure\n");

            }
            break;

        case TM_GET_GPIO_EXP_INFO:
            {
                /* Decode Location information */
                memcpy(&gpio_exp_inf,tm_frame.tm_data, sizeof(gpio_exp_info));
#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",gpio_exp_inf.instance_id);
                printf("intr_pin_polarity = %d \n",gpio_exp_inf.intr_pin_polarity);
#endif
            }
            break;

        case TM_SET_DIG_THERM_PARM:
            {
                memcpy(&set_dig_therm_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_dig_therm_parm_sts)
                    printf("Set digital thermostat parameter success\n");
                else
                    printf("Set digital thermostat parameter failure\n");

            }
            break;

        case TM_GET_DIG_THERM_INFO:
            {
                /* Decode Location information */
                memcpy(&dgtl_thrmst_inf,tm_frame.tm_data, sizeof(digital_thermostat_info));
#ifdef SNS_GET_INF_PRINT
                printf("instance_id=%d \n",dgtl_thrmst_inf.instance_id);
                printf("high_temp_thrsld=%d\n",dgtl_thrmst_inf.high_temp_thrsld);
                printf("low_temp_thrsld=%d\n",dgtl_thrmst_inf.low_temp_thrsld);
                printf("po_pin_logic=%d\n",dgtl_thrmst_inf.po_pin_logic);
                printf("adc_resolution=%d\n",dgtl_thrmst_inf.adc_resolution);
                printf("temperature=%d\n",dgtl_thrmst_inf.temperature);
#endif
            }
            break;

        case TM_SET_PWR_MON_PARM:
            {
                memcpy(&set_pwr_mon_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_pwr_mon_parm_sts)
                    printf("Set power monitor parameter success\n");
                else
                    printf("Set power monitor parameter failure\n");

            }
            break;

        case TM_GET_PWR_MON_INFO:
            {
                /* Decode Location information */
                memcpy(&pwr_sns_mtr_inf,tm_frame.tm_data, sizeof(power_sense_mntr_info));
#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",pwr_sns_mtr_inf.instance_id);
                printf("vbus_thrsld = %d \n",pwr_sns_mtr_inf.vbus_thrsld);
                printf("vshunt_thrsld = %d \n",pwr_sns_mtr_inf.vshunt_thrsld);
                printf("calibration = %d \n",pwr_sns_mtr_inf.calibration);
                printf("power = %d \n",pwr_sns_mtr_inf.power);
#endif
            }
            break;

        case TM_SET_VOLT_CTRL_MON_PARM:
            {
                memcpy(&set_volt_ctrl_mon_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_volt_ctrl_mon_parm_sts)
                    printf("Set voltage control and monitor parameter success\n");
                else
                    printf("Set voltage control and monitor parameter failure\n");

            }
            break;

        case TM_GET_VOLT_CTRL_MON_INFO:
            {
                /* Decode Location information */
                memcpy(&htswp_cntlr_inf,tm_frame.tm_data, sizeof(hotswap_cntlr_info));
#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",htswp_cntlr_inf.instance_id);
                printf("voltage = %f \n",htswp_cntlr_inf.voltage);
                printf("current = %f \n",htswp_cntlr_inf.current);
                printf("ovr_crnt_sts = %d \n",htswp_cntlr_inf.ovr_crnt_sts);
                printf("hotswap_sts = %d \n",htswp_cntlr_inf.hotswap_sts);
#endif
            }
            break;

        case TM_SET_PWR_SEQ_MON_PARM:
            {
                memcpy(&set_pwr_seq_mon_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_pwr_seq_mon_parm_sts)
                    printf("Set power sequence and monitor parameter success\n");
                else
                    printf("Set power sequence and monitor failure\n");
            }
            break;

        case TM_GET_PWR_SEQ_MON_INFO:
            {
                /* Decode Location information */
                memcpy(&vltg_seq_mntr_inf,tm_frame.tm_data, sizeof(voltage_seq_mntr_info));
#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",vltg_seq_mntr_inf.instance_id);
                printf("flash_lock_sts = %d \n",vltg_seq_mntr_inf.flash_lock_sts);
                printf("v_ref_type = %d \n",vltg_seq_mntr_inf.v_ref_type);
#endif
            }
            break;

        case TM_SET_DATA_LOG_PARM:
            {
                memcpy(&set_data_log_parm_sts,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_data_log_parm_sts)
                    printf("Set data logger parameter success\n");
                else
                    printf("Set data logger parameter failure\n");

            }
            break;

        case TM_GET_DATA_LOG_INFO:
            {
                /* Decode Location information */
                memcpy(&data_acq_inf,tm_frame.tm_data, sizeof(data_acq_dvc_info));
#ifdef SNS_GET_INF_PRINT
                printf("instance_id = %d \n",data_acq_inf.instance_id);
                printf("chnl_num = %d \n",data_acq_inf.chnl_num);
                printf("pwr_dwn_optn = %d \n",data_acq_inf.pwr_dwn_optn);
                printf("data = %d \n",data_acq_inf.data);
#endif
            }
            break;

			        case TM_SET_SPI_INTF_PARM:
            {
                memcpy(&spi_intf_param_rsp_sts,tm_frame.tm_data,4);
                if(!spi_intf_param_rsp_sts)
                    printf("spi intf param success\n");
                else
                    printf("spi intf param failure\n");
            }
            break;

        case TM_SET_ETH_INTF_PARM:
            {
                memcpy(&eth_intf_param_rsp_sts,tm_frame.tm_data,4);
                if(!eth_intf_param_rsp_sts)
                    printf("Ethernet intf param success\n");
                else
                    printf("Ethernet intf param failure\n");
            }
            break;
        case TM_SET_CAN_INTF_PARM:
            {
                memcpy(&can_intf_param_rsp_sts,tm_frame.tm_data,4);
                if(!can_intf_param_rsp_sts)
                    printf("CAN intf param success\n");
                else
                    printf("CAN intf param failure\n");
            }
            break;
        case TM_SET_I2C_INTF_PARM:
            {
                memcpy(&i2c_intf_param_rsp_sts,tm_frame.tm_data,4);
                if(!i2c_intf_param_rsp_sts)
                    printf("I2C intf param success\n");
                else
                    printf("I2C intf param failure\n");
            }
            break;
        case TM_SET_USB_INTF_PARM:
            {
                memcpy(&usb_intf_param_rsp_sts,tm_frame.tm_data,4);
                if(!usb_intf_param_rsp_sts)
                    printf("USB intf param success\n");
                else
                    printf("USB intf param failure\n");
            }
            break;
        case TM_SET_UART_INTF_PARM:
            {
                memcpy(&uart_intf_param_rsp_sts,tm_frame.tm_data,4);
                if(!uart_intf_param_rsp_sts)
                    printf("UART intf param success\n");
                else
                    printf("UART intf param failure\n");
            }
            break;
        case TM_GET_SPI_INTF_PARM:
            {

                tm_spi_intf_param_dec(tm_frame.tm_data, &spii);
            }
            break;
        case TM_GET_ETH_INTF_PARM:
            {

                tm_ethernet_init_param_dec(tm_frame.tm_data, &eth);
            }
            break;
        case TM_GET_CAN_INTF_PARM:
            {

                tm_can_init_param_dec(tm_frame.tm_data, &cis);
            }
            break;
        case TM_GET_I2C_INTF_PARM:
            {

                tm_i2c_init_param_dec(tm_frame.tm_data, &i2ci);
            }
            break;
        case TM_GET_USB_INTF_PARM:
            {

                tm_usb_init_param_dec(tm_frame.tm_data, &usbo);
            }
            break;
        case TM_GET_UART_INTF_PARM:
            {

                tm_uart_init_param_dec(tm_frame.tm_data, &uarti);
            }
            break;

        case TM_SET_CONV_BOARD_PARM:
            {
                memcpy(&set_conv_brd_par,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_conv_brd_par)
                    printf("Cnvertor board parameter configuration success\n");
                else
                    printf("Convertor board parameter configuration failure\n");
            }
            break;

        case TM_SET_BATT_BOARD_PARM:
            {
                memcpy(&set_batt_brd_par,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_batt_brd_par)
                    printf("Battery board parameter configuration success\n");
                else
                    printf("Battery board parameter configuration failure\n");
            }
            break;

        case TM_SET_OUT_CHNL_ENB:
            {
                memcpy(&out_chnl_enb,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!out_chnl_enb)
                    printf("Output channel enable configuration success\n");
                else
                    printf("Output channel enable configuration failure\n");
            }
            break;

        case TM_SET_OUT_CHNL_CUR_LMT:
            {
                memcpy(&out_chnl_cur_lmt,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!out_chnl_cur_lmt)
                    printf("Output channel current limit configuration success\n");
                else
                    printf("Output channel current limit configuration failure\n");
            }
            break;

        case TM_SET_EPS_MODE_CFG:
            {
                memcpy(&set_eps_mode,tm_frame.tm_data,4);
                /* Check target pointing configuration response status */
                if(!set_eps_mode)
                    printf("EPS Mode configuration success\n");
                else
                    printf("EPS Mode configuration failure\n");
            }
            break;
        case TM_GET_CONV_BOARD_PARM:
            {
                /* Decode Location information */
                tm_get_conv_brd_dec(tm_frame.tm_data, &brd);
            }
            break;
        case TM_GET_BATT_BOARD_PARM:
            {
                /* Decode Location information */
                tm_get_batt_brd_dec(tm_frame.tm_data, &batt);
            }
            break;
        case TM_GET_OUT_CHNL_PARM:
            {
                /* Decode Location information */
                tm_get_out_chnl_par_dec(tm_frame.tm_data, &chnl);
            }
            break;
        case TM_GET_EPS_MODE_CFG:
            {
                /* Decode Location information */
                tm_get_eps_mode_dec(tm_frame.tm_data, &eps);
            }
            break;


        case TM_FTCI:
        case TM_FTFCI:
        case TM_FTSR:
        case TM_FTDS:
        	ft_destination_set(tm_src_id);
        	ft_payload_parser(tm_id,tm_frame.tm_data,tm_frame.tm_len);
			break;


        case TM_SET_COMMS_TX_CFG:
            {
                os_memcpy(&sts,tm_frame.tm_data,tm_len);
                /* Check Set TX configuration configuration response status */

                if(!sts)
                {
                  printf("MFCC APP: Set TX config is success \n");
                }
                else
                {
                    printf("MFCC APP: Set TX config failure \n");
                }

            }
            break;

        case TM_GET_COMMS_TX_CFG:
            {
                os_memcpy(&get_tx_cfg,tm_frame.tm_data,tm_len);
                printf("MFCC APP: Transmit frequency : %d\n",
                       get_tx_cfg.tx_cfg_freq);
                printf("MFCC APP: Transmit modulation type : %d\n",
                       get_tx_cfg.tx_cfg_type);
                printf("MFCC APP: Transmit symbol rate : %d\n",
                       get_tx_cfg.tx_cfg_rate);
                printf("MFCC APP: RRC roll-off : %d\n",
                       get_tx_cfg.tx_cfg_rolloff);

            }
            break;

        case TM_SET_COMMS_RX_CFG:
            {
                os_memcpy(&sts,tm_frame.tm_data,tm_len);
                if(!sts)
                {
                  printf("MFCC APP: Set RX config is success \n");
                }
                else
                {
                    printf("MFCC APP:Set RX config failure \n");
                }
            }
            break;

        case TM_GET_COMMS_RX_CFG:
            {
                os_memcpy(&get_rx_cfg,tm_frame.tm_data,tm_len);
                printf("MFCC APP: Transmit frequency : %d\n",
                       get_rx_cfg.rx_cfg_freq);
                printf("MFCC APP: Transmit modulation type : %d\n",
                       get_rx_cfg.rx_cfg_type);
                printf("MFCC APP: Transmit symbol rate : %d\n",
                       get_rx_cfg.rx_cfg_rate);
                printf("MFCC APP: RRC roll-off : %d\n",
                       get_rx_cfg.rx_cfg_rolloff);
            }
            break;

        case TM_SET_COMMS_SYS_CFG:
            {
                os_memcpy(&sts,tm_frame.tm_data,tm_len);
                if(!sts)
                {
                  printf("MFCC APP: Set SYS config is success \n");
                }
                else
                {
                    printf("MFCC APP: Set SYS config failure \n");
                }
            }
            break;

        case TM_GET_COMMS_SYS_CFG:
            {
                os_memcpy(&get_sys_cfg,tm_frame.tm_data,tm_len);
                printf("MFCC APP: system String name : %s\n",
                       get_sys_cfg.sys_name);
                printf("MFCC APP: Serial number/ID : %d\n",
                       get_sys_cfg.sys_serial);
                printf("MFCC APP: Running software version : %d\n",
                       get_sys_cfg.sys_sw_version);
                printf("MFCC APP: Boot count : %d\n",
                       get_sys_cfg.sys_bootcount);
            }
            break;

        default :
            {
                printf("Invalid TM ID received %d\n",tm_id);
            }
    }
    /* Free allocated memories */
    free(tm_frame.tm_data);
    if(ctrl_param.encrypt)
    {
        free(param.key);
        free(tm_frame.cmac_hmac);
    }

    return 0;
}

/**
 * @brief Function that validates sequence number
 */
void mfcc_validate_seq_num(uint16 actual_seq_no)
{
    static uint16 expected_seq_no = 1;
    uint32 missing_frames , i = 0;
    if(expected_seq_no != actual_seq_no)
    {
        if(actual_seq_no > expected_seq_no)
        {
            missing_frames = actual_seq_no - expected_seq_no;
            for (i=0; i<missing_frames; i++)
            {
                printf("Warning !!! Missing frame. Sequence number : %d \n",expected_seq_no);
                expected_seq_no++;
            }
        }
        else
        {
           printf(" Warning !!! Received the same packet again : %d \n",actual_seq_no);
        }

    }
    expected_seq_no++;
}
/*********************************************************************************/
/******************* TeleCommand Encode Functions ********************************/
/*********************************************************************************/

/*!
 * @brief This API encodes the Interface Control Messsage for
 * Target point request
 */
int8 tc_tty_data_enc(uint8* data, uint8* tc_tty_data, uint32 data_size)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,tc_tty_data,data_size);

    status = TCTM_OK;

    return status;
}

int8 tc_trgt_point_req_enc(uint8* data, tctm_trgt_pt_req_s *tpr)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,&tpr->lat,4); data += 4;
    memcpy(data,&tpr->longit,4); data += 4;
    memcpy(data,&tpr->alt,4); data += 4;
    *data = tpr->pld_hw_id; data += 1;
    *data = tpr->act_tmr_min; data += 1;
    *data = tpr->act_tmr_sec; data += 1;
    memcpy(data,&tpr->act_tmr_ms,2);

    status = TCTM_OK;

    return status;
}

int8 tc_schedule_deploy_enc(uint8 *data, uint8 num_days)
{
    int8 status;
    orbit_lut_cfg_s orb_lut_cfg;

    *data = num_days; data++;

    orb_lut_cfg.date = 31;
    orb_lut_cfg.month = 10;
    orb_lut_cfg.year = 2022;
    orb_lut_cfg.orbit_lut_map[0] = 1;
    orb_lut_cfg.orbit_lut_map[1] = 1;
    orb_lut_cfg.orbit_lut_map[2] = 1;
    orb_lut_cfg.orbit_lut_map[3] = 1;
    orb_lut_cfg.orbit_lut_map[4] = 1;
    orb_lut_cfg.orbit_lut_map[5] = 1;
    orb_lut_cfg.orbit_lut_map[6] = 1;
    orb_lut_cfg.orbit_lut_map[7] = 1;
    orb_lut_cfg.orbit_lut_map[8] = 1;
    orb_lut_cfg.orbit_lut_map[9] = 1;
    orb_lut_cfg.orbit_lut_map[10] = 1;
    orb_lut_cfg.orbit_lut_map[11] = 1;
    orb_lut_cfg.orbit_lut_map[12] = 1;
    orb_lut_cfg.orbit_lut_map[13] = 1;
    orb_lut_cfg.orbit_lut_map[14] = 1;
    orb_lut_cfg.orbit_lut_map[15] = 1;
    memcpy(data,&orb_lut_cfg,sizeof(orbit_lut_cfg_s));
    data += sizeof(orbit_lut_cfg_s);

    orb_lut_cfg.date = 31;
    orb_lut_cfg.month = 10;
    orb_lut_cfg.year = 2022;
    orb_lut_cfg.orbit_lut_map[0] = 0;
    orb_lut_cfg.orbit_lut_map[1] = 0;
    orb_lut_cfg.orbit_lut_map[2] = 0;
    orb_lut_cfg.orbit_lut_map[3] = 0;
    orb_lut_cfg.orbit_lut_map[4] = 0;
    orb_lut_cfg.orbit_lut_map[5] = 0;
    orb_lut_cfg.orbit_lut_map[6] = 0;
    orb_lut_cfg.orbit_lut_map[7] = 2;
    orb_lut_cfg.orbit_lut_map[8] = 0;
    orb_lut_cfg.orbit_lut_map[9] = 0;
    orb_lut_cfg.orbit_lut_map[10] = 1;
    orb_lut_cfg.orbit_lut_map[11] = 0;
    orb_lut_cfg.orbit_lut_map[12] = 0;
    orb_lut_cfg.orbit_lut_map[13] = 0;
    orb_lut_cfg.orbit_lut_map[14] = 0;
    orb_lut_cfg.orbit_lut_map[15] = 0;
    memcpy(data,&orb_lut_cfg,sizeof(orbit_lut_cfg_s));
    status = TCTM_OK;

    return status;
}

int8 tc_schedule_cfg_lut_enc(uint8 *data, uint16 num_events)
{
    int8 status;
    uint8 lut_id;
    schedule_event_s event_info;

    lut_id = 1;

    *data = lut_id; data++;
    memcpy(data,&num_events,sizeof(num_events)); data += sizeof(num_events);

    ////////////////////////////////////
    //////IoT-Mode A
    /////////////////////////////////
    event_info.event_id = ADCS_POINT;
    event_info.event_exec_time.min = 1;
    event_info.event_exec_time.sec = 10;
    event_info.event_arg = INDONESIAN_BRDR_LAT;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = POWER_ON_PLD;
    event_info.event_exec_time.min = 1;
    event_info.event_exec_time.sec = 40;
    event_info.event_arg = 1;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = BOOT_UP_PLD_VM;
    event_info.event_exec_time.min = 2;
    event_info.event_exec_time.sec = 0;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = NETRA_IOT_MODE_A;
    event_info.event_exec_time.min = 2;
    event_info.event_exec_time.sec = 20;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = SHUTDOWN_PLD_VM;
    event_info.event_exec_time.min = 2;
    event_info.event_exec_time.sec = 50;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    ////////////////////////////////////
    //////IoT-Mode B
    /////////////////////////////////
    event_info.event_id = ADCS_POINT;
    event_info.event_exec_time.min = 3;
    event_info.event_exec_time.sec = 10;
    event_info.event_arg = INDONESIAN_BRDR_LAT;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = POWER_ON_PLD;
    event_info.event_exec_time.min = 3;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = 1;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = BOOT_UP_PLD_VM;
    event_info.event_exec_time.min = 3;
    event_info.event_exec_time.sec = 50;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = NETRA_IOT_MODE_B;
    event_info.event_exec_time.min = 4;
    event_info.event_exec_time.sec = 10;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = SHUTDOWN_PLD_VM;
    event_info.event_exec_time.min = 4;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    ////////////////////////////////////
    //////Netra Firmware update
    /////////////////////////////////
    event_info.event_id = POWER_ON_PLD;
    event_info.event_exec_time.min = 4;
    event_info.event_exec_time.sec = 55;
    event_info.event_arg = 1;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = BOOT_UP_PLD_VM;
    event_info.event_exec_time.min = 5;
    event_info.event_exec_time.sec = 40;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = NETRA_UPGRADE_FW;
    event_info.event_exec_time.min = 6;
    event_info.event_exec_time.sec = 0;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = SHUTDOWN_PLD_VM;
    event_info.event_exec_time.min = 6;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    ////////////////////////////////////
    //////Netra APP update
    /////////////////////////////////
    event_info.event_id = POWER_ON_PLD;
    event_info.event_exec_time.min = 7;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = 1;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = BOOT_UP_PLD_VM;
    event_info.event_exec_time.min = 8;
    event_info.event_exec_time.sec = 0;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = NETRA_UPGRADE_APP;
    event_info.event_exec_time.min = 8;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = SHUTDOWN_PLD_VM;
    event_info.event_exec_time.min = 9;
    event_info.event_exec_time.sec = 0;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    ////////////////////////////////////
    //////TransCelestial Campaign 0
    /////////////////////////////////
    event_info.event_id = ADCS_POINT;
    event_info.event_exec_time.min = 9;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = INDONESIAN_BRDR_LAT;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = POWER_ON_PLD;
    event_info.event_exec_time.min = 9;
    event_info.event_exec_time.sec = 50;
    event_info.event_arg = 2;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = BOOT_UP_PLD_VM;
    event_info.event_exec_time.min = 10;
    event_info.event_exec_time.sec = 10;
    event_info.event_arg = 11;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = TC_CAMPAIGN0;
    event_info.event_exec_time.min = 10;
    event_info.event_exec_time.sec = 30;
    event_info.event_arg = 11;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);

    event_info.event_id = SHUTDOWN_PLD_VM;
    event_info.event_exec_time.min = 10;
    event_info.event_exec_time.sec = 50;
    event_info.event_arg = 10;
    memcpy(data,&event_info,sizeof(event_info)); data += sizeof(event_info);
    status = TCTM_OK;

    return status;
}

int8 tc_set_conv_brd_par_enc(uint8_t* data,tctm_set_conv_board_par_s *board)
{
    int8 status;
    memcpy(data,board,sizeof(tctm_set_conv_board_par_s));
    printf("convertor board: %d\n",board->mppt_conv1_enb);
    printf("convertor board: %d\n",board->mppt_conv2_enb);
    printf("convertor board:%d\n",board->mppt_conv3_enb);
    printf("convertor board: %d\n",board->mppt_conv4_enb);
    status = TCTM_OK;

    return status;
}

int8 tc_set_batt_brd_par_enc(uint8_t* data,tctm_set_batt_board_par_s *batt)
{
    int8 status;
    memcpy(data,batt,sizeof(tctm_set_batt_board_par_s));
    printf("Battery board: %f\n",batt->bat_cur_pos);
    printf("Battery  board: %f\n",batt->bat_cur_neg);
    printf("Battery  board: %d\n",batt->batt1_temp_low_lmt_val);
    printf("Battery  board: %d\n",batt->batt1_temp_upp_lmt_val);

    status = TCTM_OK;

    return status;
}

int8 tc_out_chnl_enb_enc(uint8_t* data,tctm_out_chnl_enb_s *chnl)
{
    int8 status;
    memcpy(data,chnl,sizeof(tctm_out_chnl_enb_s));
    printf("Output channel: %d\n",chnl->ch1_enb);
    printf("Output channel: %d\n",chnl->ch2_enb);
    printf("Output channel: %d\n",chnl->ch3_enb);
    printf("Output channel: %d\n",chnl->ch4_enb);

    status = TCTM_OK;

    return status;
}

int8 tc_out_chnl_cur_lmt_enc(uint8_t* data,tctm_out_chnl_cur_lmt_s *cur)
{
    int8 status;
    memcpy(data,cur,sizeof(tctm_out_chnl_cur_lmt_s ));
    printf("output channel current limit: %f\n",cur->ch1_12v_cur_val);
    printf("output channel current limit: %f\n",cur->ch2_12v_cur_val);
    printf("output channel current limit: %f\n",cur->ch3_5v_cur_val);
    printf("output channel current limit: %f\n",cur->ch4_12v_cur_val);

    status = TCTM_OK;

    return status;
}

int8 tc_set_eps_mode_cfg_enc(uint8_t* data,tctm_eps_mode_par_s *eps)
{
    int8 status;
    memcpy(data,eps,sizeof(tctm_eps_mode_par_s));
    printf("EPS Mode: %f\n",eps->nrml_mde_thrld_vol_val);
    printf("EPS Mode: %f\n",eps->safe_mde_thrld_vol_val);
    printf("EPS Mode: %f\n",eps->crtl_mde_thrld_vol_val);
    printf("EPS Mode: %d\n",eps->full_mde_ch1_enb);

    status = TCTM_OK;

    return status;
}

int8 tc_spi_intf_param_enc(uint8* data, tctm_spi_init_s *spii)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,spii,sizeof(tctm_spi_init_s));
    status = TCTM_OK;

    return status;
}

int8 tc_ethernet_init_param_enc(uint8* data, tctm_ethernet_init_s *eth)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,eth,sizeof(tctm_ethernet_init_s));
    status = TCTM_OK;

    return status;
}

int8 tc_can_init_param_enc(uint8* data, tctm_can_init_s *cis)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,cis,sizeof(tctm_can_init_s));
    status = TCTM_OK;

    return status;
}

int8 tc_i2c_init_param_enc(uint8* data, tctm_i2c_init_s *i2ci)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,i2ci,sizeof(tctm_i2c_init_s));
    status = TCTM_OK;

    return status;
}

int8 tc_usb_init_param_enc(uint8* data, tctm_usb_otg_cfg_s *usbo)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,usbo,sizeof(tctm_usb_otg_cfg_s));
    status = TCTM_OK;

    return status;
}

int8 tc_uart_init_param_enc(uint8* data, tctm_uart_init_s *uarti)
{
    int8 status;

    /* Encode data parameters */
    memcpy(data,uarti,sizeof(tctm_uart_init_s));
    status = TCTM_OK;

    return status;
}


/*********************************************************************************/
/******************* TeleMetry Decode Functions ********************************/
/*********************************************************************************/

/*!
 * @brief This API decodes the TM for GNNS location
 * sharing information
 */
int8 tm_gnss_loc_info_dec(uint8* data, tctm_gnss_loc_info_s *gli)
{
    int8 status;

    /* Decode data parameters */
    gli->status = *data; data += 4;
    memcpy(&gli->lat,data,4); data += 4;
    memcpy(&gli->longit,data,4); data += 4;
    memcpy(&gli->alt,data,4);
    if(!gli->status)
    {
        printf("GNSS Location Information Received Successfully\n");
        printf("Latitude : %f\n",gli->lat);
        printf("Longitude : %f\n",gli->longit);
        printf("Altitude : %f\n",gli->alt);
    }
    else
    {
        printf("GNSS Location Information Receiption Failure \n");
    }

    status = TCTM_OK;
    return status;
}

int8 tm_tty_data_dec(uint8* data, uint8* tm_tty_data, uint32 data_size)
{
    int8 status;

    /* Decode data parameters */
    memcpy(tm_tty_data,data,data_size);

    status = TCTM_OK;
    return status;
}
int8 tm_spi_intf_param_dec(uint8* data, tctm_spi_init_s *spii)
{
    int8 status;

    /* Decode data parameters */
    memcpy(&spii,data,sizeof(spii));
    printf("SPI mode Received Successfully\n");

    status = TCTM_OK;
    return status;
}

int8 tm_ethernet_init_param_dec(uint8* data, tctm_ethernet_init_s *eth)
{
    int8 status;

    /* Decode data parameters */
    memcpy(&eth,data,sizeof(eth));
    printf("Ethernet speed Received Successfully\n");
    status = TCTM_OK;
    return status;
}

int8 tm_can_init_param_dec(uint8* data, tctm_can_init_s *cis)
{
    int8 status;

    /* Decode data parameters */
    memcpy(&cis,data,sizeof(cis));
    printf("CAN prescaler Received Successfully\n");
    status = TCTM_OK;
    return status;
}

int8 tm_i2c_init_param_dec(uint8* data, tctm_i2c_init_s *i2ci)
{
    int8 status;

    /* Decode data parameters */
    memcpy(&i2ci,data,sizeof(i2ci));
    printf("I2C Timing Received Successfully\n");
    status = TCTM_OK;
    return status;
}

int8 tm_usb_init_param_dec(uint8* data, tctm_usb_otg_cfg_s *usbo)
{
    int8 status;

    /* Decode data parameters */
    memcpy(&usbo,data,sizeof(usbo));
    printf("USB Dev_endpoints Received Successfully\n");

    status = TCTM_OK;
    return status;
}

int8 tm_uart_init_param_dec(uint8* data, tctm_uart_init_s *uarti)
{
    int8 status;

    /* Decode data parameters */
    memcpy(&uarti,data,sizeof(uarti));
    printf("UART baudrateReceived Successfully\n");
    status = TCTM_OK;
    return status;
}

int8 tm_get_conv_brd_dec(uint8* data, tctm_get_conv_board_par_s *board)
{
    int8 status;

    /* Decode data parameters */
    memcpy(board,data,sizeof(tctm_get_conv_board_par_s));
    printf("Received Convertor board configuration Information \n");
    printf("mppt_conv1_enb value is %d\n",board->mppt_conv1_enb);
    printf("mppt_conv2_enb value is %d\n",board->mppt_conv2_enb);
    printf("mppt_conv3_enb value is %d\n",board->mppt_conv3_enb);
    printf("mppt_conv4_enb value is %d\n",board->mppt_conv4_enb);

    status = TCTM_OK;
    return status;
}



int8 tm_get_batt_brd_dec(uint8* data, tctm_get_batt_board_par_s *batt)
{
    int8 status;

    /* Decode data parameters */
    memcpy(batt,data,sizeof(tctm_get_batt_board_par_s));
    printf("Received Battery board configuration Information \n");
    printf("bat_cur_pos value is %f\n",batt->bat_cur_pos);
    printf("bat_cur_neg value is %f\n",batt->bat_cur_neg);
    printf("batt1_temp_low_lmt_val is %d\n",batt->batt1_temp_low_lmt_val);
    printf("batt1_temp_upp_lmt_val is %d\n",batt->batt1_temp_upp_lmt_val);

    status=TCTM_OK;
    return status;
}


int8 tm_get_out_chnl_par_dec(uint8* data, tctm_get_out_chnl_par_s *chnl)
{
    int8 status;

    /* Decode data parameters */
    memcpy(chnl,data,sizeof(tctm_get_out_chnl_par_s));
    printf("Received Output Channel configuration Information \n");
    printf("ch1_enb value is %d\n",chnl->ch1_enb);
    printf("ch2_enb value is %d\n",chnl->ch2_enb);
    printf("ch3_enb value is %d\n",chnl->ch3_enb);
    printf("ch4_enb value is %d\n",chnl->ch4_enb);

    status=TCTM_OK;
    return status;
}

int8 tm_get_eps_mode_dec(uint8* data, tctm_eps_mode_par_s *eps)
{
    int8 status;

    /* Decode data parameters */
    memcpy(eps,data,sizeof(tctm_eps_mode_par_s));
    printf("Received EPS Mode Information \n");
    printf("nrml_mde_thrld_vol_val value is %f\n",eps->nrml_mde_thrld_vol_val);
    printf("safe_mde_thrld_vol_val value is %f\n",eps->safe_mde_thrld_vol_val);
    printf("crtl_mde_thrld_vol_val value is %f\n",eps->crtl_mde_thrld_vol_val);
    printf("full_mde_ch1_enb value is %d\n",eps->full_mde_ch1_enb);

    status=TCTM_OK;
    return status;
}

int32_t mfcc_tcp_send(uint32_t addr, uint32_t port, uint8_t *data, uint32_t size)
{
    int32_t status;
    int sockfd; // connfd;
    struct sockaddr_in servaddr; // cli;

    // socket create and verification
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd == -1) {
        printf("socket creation failed...\n");
        status = sockfd;
    }
    else
    {
        bzero(&servaddr, sizeof(servaddr));

        // assign IP, PORT
        servaddr.sin_family = AF_INET;
        servaddr.sin_addr.s_addr = addr;
        servaddr.sin_port = htons(port);

        //printf(" connecting to server ..");
        // connect the client socket to server socket
        while (connect(sockfd, (struct sockaddr*)&servaddr, sizeof(servaddr)) < 0)
        {

            //printf(".");
            /* wait for server */

        }

        //printf("SOCKET: connected to the server..\n");
        status = write(sockfd, data, size);
        printf("[SOCKET_TCTM] Successfully transferred data of size %d\n", status);
        status = read(sockfd, data, 1024);
        printf("[SOCKET_TCTM] Successfully received data of size %d \n",status);
        if(status>0)
        {
            mfcc_tm_parser(data);
        }
        // close the socket
        close(sockfd);
    }
    return status;
}
